/*
 ************************************************************************************************
 *                                                                                              
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 
 *                                                                                              
 *  OSA is distributed under BSD license (see license.txt)                                      
 *                                                                                              
 *  URL:        http://wiki.pic24.ru/doku.php/en/osa/ref/intro                                  
 *              http://picosa.narod.ru                                                          
 *                                                                                              
 *----------------------------------------------------------------------------------------------
 *                                                                                              
 *  File:       osa.c                                                                           
 *                                                                                              
 *  Compilers:  HT-PICC STD
 *              HT-PICC PRO
 *              HT-PICC18 STD
 *              Microchip C18
 *              Microchip C30
 *              MikroC
 *              CCS
 *              WinAVR
 *              IAR
 *
 *                                                                                              
 *  Programmer: Timofeev Victor                                                                 
 *              osa@pic24.ru, testerplus@mail.ru                                                
 *                                                                                              
 *  Definition: This file contains all system variables and functions definitions.              
 *                                                                                              
 *  History:    21.01.2009                                                                      
 *              02.04.2009 -    Corrected condition "#if OST_CSEM_SIZE > 1" : '>' -> '!='       
 *                              Corrected condition "#if OST_SMSG_SIZE > 1" : '>' -> '!='       
 *              01.04.2009      Precessor specific definitions placed in processor's c-file     
 *                 07.2009 -    1. Some modifications to _OS_Task_Create to make it universal   
 *                              2. Dtimer's functions moved from mcc18 and picc18 here          
 *                              3. Clearing of _OS_DTimes->Next field added into OS_Init        
 *              29.10.2009 -    Type 'bit' replaced with __osa_bool for _OS_CheckSmsg and       
 *                              _OS_CheckEvent                                                  
 *              03.11.2009 -    Bug fixed: Added bReady=1 in OS_Check_Event when bTimeout       
 *              04.11.2009 -    All OS_SMsg_xxx renamed to OS_Smsg_xxx
 *              10.12.2009 -    1. All services "SQueue" renamed to "Squeue"
 *                              2. Added services OS_Squeue_Get_I and OS_Queue_Get_I
 *                              4. __OS_QUEUE_DI and __OS_QUEUE_RI were removed from _OS_xxx_Send
 *                              5. Service _OS_Csem_Signal_I added
 *                              6. Function _OS_Smsg_Check was removed
 *                              7. Function _OS_Csem_Check was removed
 *                              8. Assignment in OS_Init reencoded to avoid warning
 * 
 *              11.03.2010 -    6. Clearing "priority" removed from OS_Init
 *                              7. _OS_Stimers is volatile now
 * 
 *                                                                                              
 ************************************************************************************************
 */




#include <OSA.h>





/*
 ************************************************************************************************
 *                                                                                              *
 *  V A R I A B L E S   D E F I N I T I O N                                                     *
 *                                                                                              *
 ************************************************************************************************
 */

//------------------------------------------------------------------------------
#ifdef __OSAMCC18__
#pragma udata access ACCESS
#endif
//------------------------------------------------------------------------------



NEAR    OST_SYSTEM_FLAGS             OS_Flags           __osa30_near;   // System state flags
NEAR   __osa_word                   _OS_temp            __osa30_near;   // For internal purpose

#ifdef OS_ENABLE_INT
NEAR   __osa_word                   _OS_temp_I          __osa30_near;
#endif

#ifndef __OSAPICC12__
NEAR   __osa_word                   _OS_tempH           __osa30_near;   // For internal purpose
#endif


#if defined(_OS_CUR_FLAGS_IN_OS_state)
NEAR    OST_TASK_STATE               OS_state           __osa30_near;

#endif

        OST_TASK_POINTER    NEAR    _OS_CurTask         __osa30_near;   // Pointer to current task
                                                                        // descriptor.

//------------------------------------------------------------------------------
#if OS_TIMERS24 > 0
OS_BANK __osa_word                  OS_Ticks           __osa30_near;
#endif
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
#ifdef __OSAMCC18__
#pragma udata
#endif
//------------------------------------------------------------------------------


// dynamic timers

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------

    #if OS_BANK_OS == 0
        #ifdef __OSAMCC18__
        #pragma udata access os_bsems
        #endif
    #endif

    OS_BANK OST_DTIMER_CB               _OS_DTimers         __osa30_near;   // Pointer to list of


    #ifndef _OS_DtimersWork_DEFINED
            OST_DTIMER        * OS_BANK _OS_DtimerWork      __osa30_near;   //*100210.10*
                                                                            // Temp var for work
                                                                            // with list of 
                                                                            // dynamic timers in 
                                                                            // ISR 
    #endif

    #if OS_BANK_OS == 0
        #ifdef __OSAMCC18__
        #pragma udata
        #endif
    #endif


//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------

// fast timers

//------------------------------------------------------------------------------
/*100210.1*/ 
#ifdef OS_ENABLE_QTIMERS    
//------------------------------------------------------------------------------

    #if OS_BANK_OS == 0
        #ifdef __OSAMCC18__
        #pragma udata access os_bsems
        #endif
    #endif

    OS_BANK OST_QTIMER_CB               _OS_QTimers         __osa30_near;   // Pointer to list of
                                                                            // fast timers

    #ifndef _OS_QTimersWork_DEFINED
    OST_QTIMER                * OS_BANK _OS_QTimerWork      __osa30_near;   // Temp var for work
                                                                            // with list of 
                                                                            // fast timers in 
                                                                            // ISR 
    #endif

    #if OS_BANK_OS == 0
        #ifdef __OSAMCC18__
        #pragma udata
        #endif
    #endif

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#if !defined(OS_DISABLE_PRIORITY)
//------------------------------------------------------------------------------
        #if OS_BANK_OS == 0
            #ifdef __OSAMCC18__
            #pragma udata access os_bsems
            #endif
        #endif

        OST_TASK_POINTER    OS_BANK _OS_BestTask        __osa30_near;   // Pointer to ready task
                                                                        // with highest priority
                                                                        // (used in OS_Sched)
        OST_TASK_POINTER    OS_BANK _OS_LastTask        __osa30_near;   // Pointer to last
                                                                        // executed task
OS_BANK __osa_word                  _OS_best_priority   __osa30_near;

        #if OS_BANK_OS == 0
            #ifdef __OSAMCC18__
            #pragma udata
            #endif
        #endif

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------







/*
 *------------------------------------------*
 * Binary semaphores                        *
 *------------------------------------------*
 */
#if OS_BSEMS > 0
            #if OS_BANK_BSEM == 0
                #ifdef __OSAMCC18__
                #pragma udata access os_bsems
                #endif
            #endif

OS_BSEM_BANK    __osa_word   OS_BSem[(OS_BSEMS + _OST_INT_SIZE-1) / _OST_INT_SIZE] __osa30_near;

            #if OS_BANK_BSEM == 0
                #ifdef __OSAMCC18__
                #pragma udata
                #endif
            #endif
#endif





/*
 *------------------------------------------*
 * Task descriptors                         *
 *------------------------------------------*
 */
            #if OS_BANK_TASKS == 0
                #ifdef __OSAMCC18__
                #pragma udata access os_tasks
                #endif
            #endif

OS_TASKS_BANK   OST_TCB             OS_TaskVars[OS_TASKS]   __osa30_near OS_ALLOCATION_TASKS;

            #if OS_BANK_TASKS == 0
                #ifdef __OSAMCC18__
                #pragma udata
                #endif
            #endif




/*
 *------------------------------------------*
 * Timeouts for old style static timers     *
 *------------------------------------------*
 */

#if OS_TIMERS > 0

            #if OS_BANK_TIMEOUTS == 0
                #ifdef __OSAMCC18__
                #pragma udata access os_timeout
                #endif
            #endif

OS_TIMEOUTS_BANK __osa_word OS_Timeouts[(OS_TIMERS + _OST_INT_SIZE - 1) / _OST_INT_SIZE] __osa30_near;

            #if OS_BANK_TIMEOUTS == 0
                #ifdef __OSAMCC18__
                #pragma udata
                #endif
            #endif
#endif




/*
 *------------------------------------------*
 * 32-bit old style static timers           *
 *------------------------------------------*
 */
#if OS_TIMERS32 > 0

            #if OS_BANK_TIMERS32 == 0
                #ifdef __OSAMCC18__
                #pragma udata access os_timers32
                #endif
            #endif

OS_TIMERS32_BANK    __osa_uint32    OS_Timers32[OS_TIMERS32]    __osa30_near;

            #if OS_BANK_TIMERS32 == 0
                #ifdef __OSAMCC18__
                #pragma udata
                #endif
            #endif
#endif


/*
 *------------------------------------------*
 * 24-bit old style static timers           *
 *------------------------------------------*
 */
#if OS_TIMERS24 > 0

            #if OS_BANK_TIMERS24 == 0
                #ifdef __OSAMCC18__
                #pragma udata access os_timers24
                #endif
            #endif

OS_TIMERS24_BANK    __osa_uint16    OS_Timers24[OS_TIMERS24]    __osa30_near;

            #if OS_BANK_TIMERS24 == 0
                #ifdef __OSAMCC18__
                #pragma udata
                #endif
            #endif
#endif


/*
 *------------------------------------------*
 * 16-bit old style static timers           *
 *------------------------------------------*
 */
#if OS_TIMERS16 > 0

            #if OS_BANK_TIMERS16 == 0
                #ifdef __OSAMCC18__
                #pragma udata access os_timers16
                #endif
            #endif

OS_TIMERS16_BANK    __osa_uint16    OS_Timers16[OS_TIMERS16]    __osa30_near;

            #if OS_BANK_TIMERS16 == 0
                #ifdef __OSAMCC18__
                #pragma udata
                #endif
            #endif
#endif


/*
 *------------------------------------------*
 *  8-bit old style static timers           *
 *------------------------------------------*
 */
#if OS_TIMERS8 > 0

            #if OS_BANK_TIMERS8 == 0
                #ifdef __OSAMCC18__
                #pragma udata access os_timers8
                #endif
            #endif

OS_TIMERS8_BANK     __osa_uint8     OS_Timers8[OS_TIMERS8]      __osa30_near;

            #if OS_BANK_TIMERS8 == 0
                #ifdef __OSAMCC18__
                #pragma udata
                #endif
            #endif
#endif
//------------------------------------------------------------------------------



/*
 *------------------------------------------*
 * Static timers                            *
 *------------------------------------------*
 */
#if OS_STIMERS > 0

            #if OS_BANK_STIMERS == 0
                #ifdef __OSAMCC18__
                #pragma udata access os_stimers
                #endif
            #endif

volatile OS_STIMERS_BANK     _OST_STIMER     _OS_Stimers[OS_STIMERS]      __osa30_near; /*100311.7*/

#ifdef OS_STIMERS_ENABLE_ALLOCATION    /*100210.2*/
OS_STIMERS_BANK     __osa_word   _OS_StimersFree[(OS_STIMERS + _OST_INT_SIZE-1) / _OST_INT_SIZE];
#endif

            #if OS_BANK_STIMERS == 0
                #ifdef __OSAMCC18__
                #pragma udata
                #endif
            #endif
#endif










/*
 ************************************************************************************************
 *                                                                                              *
 *  I N C L U D E   C O M P I L E R   S P E C I F I E D   D E F I N I T I O N S                 *
 *                                                                                              *
 ************************************************************************************************
 */

#include "port\osa_include.c"




/************************************************************************************************
 *                                                                                              *
 *                                                                                              *
 *                         F U N C T I O N   D E F I N I T I O N S                              *
 *                                                                                              *
 *                                                                                              *
 ************************************************************************************************/








/*
 ************************************************************************************************
 *                                                                                              *
 *                            C O U N T I N G    S E M A P H O R E S                            *
 *                                                                                              *
 ************************************************************************************************
 */
//------------------------------------------------------------------------------
#ifdef OS_ENABLE_CSEM
//------------------------------------------------------------------------------



/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_Csem_Signal (OST_CSEM *pCSem)                                      *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  Increase counting semaphore. Set EventError if csem is FF...                *
 *                                                                              *
 *                                                                              *
 *  parameters:     pCSem - pointer to counting semaphore                       *
 *                                                                              *
 *  on return:      OS_IsEventError()                                           *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc16.c"                                         *
 *                  "port\osa_picc18.c"                                         *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if !defined(_OS_Csem_Signal_DEFINED)
//------------------------------------------------------------------------------

    void _OS_Csem_Signal (OST_CSEM *pCSem)
    {
        OS_Flags.bEventError = 0;

        (*pCSem)++;
        if (!*pCSem) 
        {
            (*pCSem) = (OST_CSEM) -1;
            OS_Flags.bEventError = 1;
        }
    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_Csem_Signal_DEFINED
//------------------------------------------------------------------------------




//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT_CSEM) && !defined(_OS_Csem_Signal_I_DEFINED)
//------------------------------------------------------------------------------

    void _OS_Csem_Signal_I (OST_CSEM *pCSem)    /*91210*/
    {
        OS_Flags.bEventError = 0;
    
        (*pCSem)++;
        if (!*pCSem) 
        {
            (*pCSem) = (OST_CSEM) -1;
            OS_Flags.bEventError = 1;
        }
    }
//------------------------------------------------------------------------------
#endif  // OS_ENABLE_INT_CSEM && !_OS_Csem_Signal_I_DEFINED
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
#endif  // OS_CSEM_ENABLE
//------------------------------------------------------------------------------
















/*
 ************************************************************************************************
 *                                                                                              *
 *                         C R I T I C A L   S E C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************
 */

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------

/*
 ********************************************************************************
 *                                                                              *
 *   void OS_EnterCriticalSection (void)                                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *   description:   Enter critical section.                                     *
 *                  This function disables interrupts (with saving current      *
 *                  state) and sets system flag bInCriticalSection              *
 *                                                                              *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc16.c"                                         *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_ccs16.c"                                          *
 *                  "port\osa_mcc30.c"                                          *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if !defined(OS_EnterCriticalSection_DEFINED)
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#ifdef      __OSA18__
//------------------------------------------------------------------------------

    void OS_EnterCriticalSection (void)
    {
        __osa_word temp;
    
        temp = OS_DI();
        OS_Flags.bInCriticalSection = 1;
    
        OS_Flags.bGIE_CTemp = 0;
        if (temp & 0x80) OS_Flags.bGIE_CTemp = 1;
    
        OS_Flags.bGIEL_CTemp = 0;
        if (temp & 0x40) OS_Flags.bGIEL_CTemp = 1;
    }


//------------------------------------------------------------------------------
#else   //  __OSA18__
//------------------------------------------------------------------------------

    void OS_EnterCriticalSection (void)
    {
                OS_Flags.bGIE_CTemp = 0;
        if (OS_DI() & 0x80) OS_Flags.bGIE_CTemp = 1;
        OS_Flags.bInCriticalSection = 1;

    }

//------------------------------------------------------------------------------
#endif  //  __OSA18__
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------




/*
 ********************************************************************************
 *                                                                              *
 *   void OS_LeaveCriticalSection (void)                                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Leave critical section.                                     *
 *                  This function restore interrupt state from OS_Flag temp bits*
 *                  and clears system flag bInCriticalSection                   *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc16.c"                                         *
 *                  "port\osa_ccs16.c"                                          *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_mcc30.c"                                          *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if !defined(OS_LeaveCriticalSection_DEFINED)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#ifdef      __OSA18__
//------------------------------------------------------------------------------

    void OS_LeaveCriticalSection (void)
    {
        char temp;
        OS_Flags.bInCriticalSection = 0;
        temp = 0;
        if (OS_Flags.bGIE_CTemp)  temp |= 0x80;
        if (OS_Flags.bGIEL_CTemp) temp |= 0x40;
        OS_RI(temp);
    }

//------------------------------------------------------------------------------
#else   //  __OSA18__
//------------------------------------------------------------------------------

    void OS_LeaveCriticalSection (void)
    {
        OS_Flags.bInCriticalSection = 0;
        if (OS_Flags.bGIE_CTemp) OS_RI(0x80);
    }

//------------------------------------------------------------------------------
#endif  //  __OSA18__
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif  //  OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------
















/*
 ************************************************************************************************
 *                                                                                              *
 *                                   D E L A Y S   I N   T A S K                                *
 *                                                                                              *
 ************************************************************************************************
 */

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_TTIMERS
//------------------------------------------------------------------------------

/*
 ********************************************************************************
 *                                                                              *
 *   void _OS_InitDelay (_OST_TTIMER Delay)                                     *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel)                 *
 *                                                                              *
 *                  Init task timer delay.Set timer variable in task descriptor,*
 *                  sets bit bDelay and clears bit bCanContinue.                *
 *                                                                              *
 *                  _OS_CurTask must point to descriptor.                       *
 *                                                                              *
 *                                                                              *
 *  parameters:     Delay   - time of delay in system ticks                     *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                  "port\osa_picc16.c"                                         *
 *                  "port\osa_ccs16.c"                                          *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_picc18.c"                                         *
 *                  "port\osa_mcc18.c"                                          *
 *                  "port\osa_mcc30.c"                                          *
 *                                                                              *
 ********************************************************************************
 */




//------------------------------------------------------------------------------
#if !defined(_OS_InitDelay_DEFINED)
//------------------------------------------------------------------------------

    void _OS_InitDelay (_OST_TTIMER Delay)
    {

        _OS_CurTask->State.bCanContinue = 0;
        _OS_CurTask->State.bDelay = 0;

        if (Delay)
        {
            Delay ^= (_OST_TTIMER)-1;
            Delay ++;
            _OS_CurTask->Timer = Delay;
            _OS_CurTask->State.bReady = 1;
            _OS_CurTask->State.bDelay = 1;
        }
    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_InitDelay_DEFINED)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif  // #ifdef OS_TIMER
//------------------------------------------------------------------------------

















/************************************************************************************************
 *                                                                                              *
 *                            S Y S T E M   F U N C T I O N S                                   *
 *                                                                                              *
 ************************************************************************************************/



/*
 ********************************************************************************
 *                                                                              *
 *   void OS_Init (void)                                                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel)                 *
 *                                                                              *
 *                  Initialize all system variables:                            *
 *                  - Clear all binary semaphores                               *
 *                  - Free all task descriptors                                 *
 *                  - Clear all timeouts for old style static timers            *
 *                  - Set _OS_LastTask and _OS_CurTask variables at last task   *
 *                    descriptor                                                *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      _OSCurTask points to last task descriptor.                  *
 *                                                                              *
 *  Overloaded in:  -                                                           *
 *                                                                              *
 ********************************************************************************
 */


void OS_Init (void)
{
    #ifdef __OSAMCC30__                 /*91210*/
    OS_Flags = (OST_SYSTEM_FLAGS){0};
    #else
    *(NEAR __osa_word*)&OS_Flags = 0;
    #endif

    OS_Init_ProcSpec();     /* Specific controller type's init operations   */



    #if !defined(OS_DISABLE_PRIORITY)

        _OS_LastTask = (OS_TASKS_BANK OST_TASK_POINTER) OS_TaskVars + (OS_TASKS - 1);

    #endif



    /*--------------------------------------*
     *                                      *
     *  Zero all old style static timers    *
     *                                      *
     *--------------------------------------*/

    #if OS_TIMERS > 11*8 || ((OS_TIMERS > 0) && defined(__OSAMCC30__))

        _OS_temp = sizeof(OS_Timeouts) / sizeof(__osa_word); /*100210.11*/
        do 
        {
            OS_Timeouts[_OS_temp-1] = 0;
        } while (--_OS_temp);

    #else
        #if OS_TIMERS > 0
            OS_Timeouts[0] = 0;
        #endif

        #if OS_TIMERS > 1*8
            OS_Timeouts[1] = 0;
        #endif

        #if OS_TIMERS > 2*8
            OS_Timeouts[2] = 0;
        #endif

        #if OS_TIMERS > 3*8
            OS_Timeouts[3] = 0;
        #endif

        #if OS_TIMERS > 4*8
            OS_Timeouts[4] = 0;
        #endif

        #if OS_TIMERS > 5*8
            OS_Timeouts[5] = 0;
        #endif

        #if OS_TIMERS > 6*8
            OS_Timeouts[6] = 0;
        #endif

        #if OS_TIMERS > 7*8
            OS_Timeouts[7] = 0;
        #endif

        #if OS_TIMERS > 8*8
            OS_Timeouts[8] = 0;
        #endif

        #if OS_TIMERS > 9*8
            OS_Timeouts[9] = 0;
        #endif

        #if OS_TIMERS > 10*8
            OS_Timeouts[10] = 0;
        #endif

    #endif


    /*--------------------------------------*
     *                                      *
     *  Clear list of dynamic timers        *
     *                                      *
     *--------------------------------------*/

    #if defined(OS_ENABLE_DTIMERS)
        *((OS_BANK __osa_word*)&_OS_DTimers.Flags) = 0;
        _OS_DTimers.Flags.bNextEnable = 0;   /*100210.5*/
        _OS_DTimers.Flags.bActive = 1;
    #endif

    /*--------------------------------------*
     *                                      *
     *  Clear list of fast timers           * 
     *                                      *
     *--------------------------------------*/

    /*100210.1*/ 
    #if defined(OS_ENABLE_QTIMERS)
        *((OS_BANK __osa_word*)&_OS_QTimers.Flags) = 0;
        // Default:
        //  bNextEnable     = 0
        //  bListFunction   = 0 (FUNCTION_ADD)
        //  bListGetTime    = 0
        _OS_QTimers.Flags.bActive = 1;
    #endif

    /*--------------------------------------*
     *                                      *
     *  Make all static timers free         * 
     *                                      *
     *--------------------------------------*/

    #if defined (OS_ENABLE_STIMERS)
    #if defined (OS_STIMERS_ENABLE_ALLOCATION)    /*100210.2*/

    #if OS_STIMERS > 11*8 || ((OS_STIMERS > 0) && defined(__OSAMCC30__))

        _OS_temp = sizeof(_OS_StimersFree) / sizeof(__osa_word); /*100210.11*/
        do 
        {
            _OS_StimersFree[_OS_temp-1] = (__osa_word)-1;
        } while (--_OS_temp);

    #else
        #if OS_STIMERS > 1*8
            _OS_StimersFree[0] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 2*8
            _OS_StimersFree[1] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 3*8
            _OS_StimersFree[2] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 4*8
            _OS_StimersFree[3] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 5*8
            _OS_StimersFree[4] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 6*8
            _OS_StimersFree[5] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 7*8
            _OS_StimersFree[6] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 8*8
            _OS_StimersFree[7] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 9*8
            _OS_StimersFree[8] = (__osa_word)-1;
        #endif

        #if OS_STIMERS > 10*8
            _OS_StimersFree[ 9] = (__osa_word)-1;
        #endif

    #endif

        #if (OS_STIMERS & _OST_INT_MASK) != 0
        _OS_StimersFree[OS_STIMERS >> _OST_INT_SHIFT] = (1 << (OS_STIMERS & _OST_INT_MASK)) - 1;
        #endif

    #endif
    #endif

    /*--------------------------------------*
     *                                      *
     *  Zero all binary semaphores          *
     *                                      *
     *--------------------------------------*/

    #if OS_BSEMS > 11*8 || ((OS_BSEMS > 0) && defined(__OSAMCC30__))

        _OS_temp = sizeof(OS_BSem) / sizeof(__osa_word);
        do 
        {
            OS_BSem[_OS_temp-1] = 0;
        } while (--_OS_temp);

    #else
        #if OS_BSEMS > 0
            OS_BSem[0] = 0;
        #endif

        #if OS_BSEMS > 1*8
            OS_BSem[1] = 0;
        #endif

        #if OS_BSEMS > 2*8
            OS_BSem[2] = 0;
        #endif

        #if OS_BSEMS > 3*8
            OS_BSem[3] = 0;
        #endif

        #if OS_BSEMS > 4*8
            OS_BSem[4] = 0;
        #endif

        #if OS_BSEMS > 5*8
            OS_BSem[5] = 0;
        #endif

        #if OS_BSEMS > 6*8
            OS_BSem[6] = 0;
        #endif

        #if OS_BSEMS > 7*8
            OS_BSem[7] = 0;
        #endif

        #if OS_BSEMS > 8*8
            OS_BSem[8] = 0;
        #endif

        #if OS_BSEMS > 9*8
            OS_BSem[9] = 0;
        #endif

        #if OS_BSEMS > 10*8
            OS_BSem[10] = 0;
        #endif
    #endif


    /*--------------------------------------*
     *                                      *
     *  Zero all task descriptors           *
     *                                      *
     *--------------------------------------*/

    #if OS_TASKS > 10
        _OS_SET_IRP_CUR_TASK(); /*90922*/
        _OS_CurTask = (OST_TASK_POINTER) OS_TaskVars;
        _OS_temp = OS_TASKS;
        do {
            _OS_CurTask->State.bEnable = 0;
            _OS_CurTask ++;
        } while (--_OS_temp);
        _OS_CLR_IRP();          /*90922*/
    #else

        #ifdef OS_DISABLE_PRIORITY
            _OS_CurTask = (OST_TASK_POINTER) OS_TaskVars + (OS_TASKS-1);
        #endif

        OS_TaskVars[0].State.bEnable = 0;

        #if OS_TASKS > 1
           OS_TaskVars[1].State.bEnable = 0;
        #endif

        #if OS_TASKS > 2
           OS_TaskVars[2].State.bEnable = 0;
        #endif

        #if OS_TASKS > 3
           OS_TaskVars[3].State.bEnable = 0;
        #endif

        #if OS_TASKS > 4
           OS_TaskVars[4].State.bEnable = 0;
        #endif

        #if OS_TASKS > 5
           OS_TaskVars[5].State.bEnable = 0;
        #endif

        #if OS_TASKS > 6
           OS_TaskVars[6].State.bEnable = 0;
        #endif

        #if OS_TASKS > 7
           OS_TaskVars[7].State.bEnable = 0;
        #endif

        #if OS_TASKS > 8
           OS_TaskVars[8].State.bEnable = 0;
        #endif

        #if OS_TASKS > 9
           OS_TaskVars[9].State.bEnable = 0;
        #endif

    #endif

}
















/*
 ********************************************************************************
 *                                                                              *
 *   void             _OS_Task_Create(__osa_word    priority,                *
 *                                    OST_CODE_POINTER TaskAddr)                *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *   description:   (Internal function called by system kernel from service     *
 *                  OS_Task_Create)                                             *
 *                  Create task in free descriptor.                             *
 *                                                                              *
 *  parameters:     priority - value from 0 (highest) to 7 (lowest)             *
 *                  TaskAddr - pointer to C-function that contains task         *
 *                                                                              *
 *  on return:      check OS_IsError                                            *
 *                  OST_TASK_POINTER - pointer to task descriptor               *
 *                  _OS_temp - task position in OS_TaskVars array  *100210.12*  *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_mcc30.c"                                          *
 *                  "port\osa_ccc16.c"                                          *
 *                                                                              *
 ********************************************************************************
 */


//------------------------------------------------------------------------------
#if !defined(_OS_Task_Create_DEFINED)
//------------------------------------------------------------------------------

    void _OS_Task_Create(__osa_word priority, OST_CODE_POINTER TaskAddr)
    {
        OST_TASK_POINTER Task;

        OS_Flags.bError = 0;

        //#if defined(OS_DISABLE_PRIORITY)
        // *100311.6* priority = 0;   // To increase scheduler speed
        //#endif



        /*--------------------------------------*
         *                                      *
         *  Start search from first task in     *
         *  OS_TASKS descriptors.               *
         *                                      *
         *--------------------------------------*/
        Task = (OST_TASK_POINTER)OS_TaskVars;
        _OS_temp = 0;   /*100210.12*/

        do {

            if (!Task->State.bEnable)               // Is descriptor free?
            {                                       // Yes
                ((OST_TASK_STATE*)&priority)->bEnable = 1;
                ((OST_TASK_STATE*)&priority)->bReady = 1;

                Task->pTaskPointer = TaskAddr;


                #ifdef OS_ENABLE_TTIMERS
                    Task->Timer = 0;
                #endif
                                
                #ifdef _OS_TASK_CREATE_PROC_SPEC                                //90700
                _OS_TASK_CREATE_PROC_SPEC;                                      //90700
                #endif                                                          //90700

                *((OS_TASKS_BANK char*)&Task->State) = priority;

                #if defined(_OS_CUR_FLAGS_IN_OS_state)                          //90700
                if (Task == _OS_CurTask) *((NEAR char*)&OS_state) = priority;   //90700
                #endif                                                          //90700

                // Now FSR points to created task descriptor
                // disabled since 90111: _fsr = (_FSR_TYPE)Task;
                
                #if defined(__OSAWINAVR__) || defined(__OSAIARAVR__)
                Task->nY_Temp = 0x8000;
                #endif

                OS_Flags.bError = 0;

                return ;

            }

            Task ++;

        } while (++_OS_temp < OS_TASKS);    /*100210.12*/

        // There is no free descriptor. Task was not created.
        OS_Flags.bError = 1;

        return ;
    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_Task_Create_DEFINED)
//------------------------------------------------------------------------------









//-----------------------------------------------------------------
#ifdef OS_ENABLE_OS_TIMER
#ifndef OS_USE_INLINE_TIMER
//-----------------------------------------------------------------
void OS_Timer (void)
{
    __OS_TimerInline();
}
//-----------------------------------------------------------------
#endif  // OS_ENABLE_OS_TIMER
#endif  // OS_USE_INLINE_TIMER
//-----------------------------------------------------------------

















/*
 ********************************************************************************
 *                                                                              *
 *  bit _OS_CheckEvent (char bEvent)                                            *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  waiting services)                                           *
 *                                                                              *
 *                  Check condition of bEvent. Accordint to bEvent value and    *
 *                  current state of bReady, bDelay and bCanContinue flags,     *
 *                  task becames ready to execute or still remains in waiting   *
 *                  mode.                                                       *
 *                                                                              *
 *  parameters:     bEvent - zero, or non-zero condition                        *
 *                                                                              *
 *  on return:      1 - when condition is true, or timeout occured              *
 *                  0 - condition is false and no timeout                       *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                  "port\osa_picc16.c"                                         *
 *                  "port\osa_picc18.c"                                         *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_mikroc18.c"                                       *
 *                  "port\osa_mcc18.c"                                          *
 *                  "port\osa_mcc30.c"                                          *
 *                  "port\osa_ccs16.c"                                          *
 *                  "port\osa_ccs18.c"                                          *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if !defined(_OS_CheckEvent_DEFINED)
//------------------------------------------------------------------------------

    __osa_bool _OS_CheckEvent (char bEvent)      // *91029*
    {
        OS_Flags.bTimeout = 0;

        if (bEvent)
        {
            if (_OS_CurTask->State.bReady)
            {
                #ifdef OS_ENABLE_TTIMERS
                    _OS_CurTask->State.bDelay = 0;
                #endif

                return 1;

            }

            _OS_bTaskReady = 1;

        } else {
            _OS_bTaskReady = 0;
        }

        #ifdef OS_ENABLE_TTIMERS

            if (!_OS_CurTask->State.bDelay && _OS_CurTask->State.bCanContinue)
            {
                _OS_bTaskReady = 1;     // *91103*
                OS_Flags.bTimeout = 1;

                return 1;

            }

        #endif

        return 0;

    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_CheckEvent_DEFINED)
//------------------------------------------------------------------------------












#include "service\osa_dtimer.c" /*100210.6*/
#include "service\osa_qtimer.c" /*100210.1*/
#include "service\osa_stimer.c" /*100210.2*/













































/************************************************************************************************
 *                                                                                              *
 *     M E S S A G E   Q U E U E S                                                              *
 *                                                                                              *
 ************************************************************************************************/


//------------------------------------------------------------------------------
#if defined(OS_ENABLE_SQUEUE) && !defined(OS_QUEUE_SQUEUE_IDENTICAL)
//------------------------------------------------------------------------------



/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_Squeue_Send (OST_SQUEUE *pSQueue, OST_SMSG SMsg)                   *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  service _OS_SendSQueue)                                     *
 *                                                                              *
 *                  Add message into queue of simple messages. Delete first     *
 *                  message if there is no free room to add new message.        *
 *                  Service OS_SQueue_Send before adding new message checks for *
 *                  free room. Thus messages will not deleted accidentally.     *
 *                                                                              *
 *                                                                              *
 *  parameters:     pSQueue      - pointer to queue descriptor                  *
 *                  SMsg        - simple message to be added                    *
 *                                                                              *
 *  on return:      OS_IsEventError() return 1, if first message was pushed out *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                                                                              *
 ********************************************************************************
 */


//------------------------------------------------------------------------------
#if !defined(_OS_Squeue_Send_DEFINED)
//------------------------------------------------------------------------------

    void _OS_Squeue_Send (OST_SQUEUE *pSQueue, OST_SMSG SMsg)
    {
        OST_QUEUE_CONTROL   q;          // Temp variable to increase speed
        __osa_word temp;

        q = pSQueue->Q;
        OS_Flags.bEventError = 0;                   // First we clear error flag


        //------------------------------------------------------
        // If there is no free room in queue, then replace
        // first message in queue by new SMsg

        if (q.cSize == q.cFilled)
        {
            pSQueue->pSMsg[q.cBegin] = SMsg;
            q.cBegin++;                             // Update pointer
            if (q.cBegin == q.cSize) q.cBegin = 0;

            OS_Flags.bEventError = 1;               // Set error flag

            goto EXIT;                              // Skip adding
        }

        //------------------------------------------------------
        // There is a free room in queue.
        // Add new message at end of queue.

        temp = q.cBegin + q.cFilled;
        if (temp >= q.cSize) temp -= q.cSize;
        pSQueue->pSMsg[temp] = SMsg;
        q.cFilled++;                                // Update fillness

    EXIT:
        pSQueue->Q = q;

    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_Squeue_Send_DEFINED)
//------------------------------------------------------------------------------






/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_Squeue_Send_I (OST_SQUEUE *pSQueue, OST_SMSG SMsg)                 *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Copy of _OS_SQueue_Send to be called from interrupt         *
 *                                                                              *
 *  parameters:     pSQueue      - pointer to queue descriptor                  *
 *                  SMsg        - simple message to be added                    *
 *                                                                              *
 *  on return:      OS_IsEventError() return 1, if first message was pushed out *
 *                                                                              *
 *  Overloaded in:  -                                                           *
 *                                                                              *
 ********************************************************************************
 */


//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT_QUEUE) && !defined(_OS_Squeue_Send_I_DEFINED)
//------------------------------------------------------------------------------

    void _OS_Squeue_Send_I (OST_SQUEUE *pSQueue, OST_SMSG SMsg)
    {
        OST_QUEUE_CONTROL   q;
        __osa_word temp;

        q = pSQueue->Q;
        OS_Flags.bEventError = 0;

        //------------------------------------------------------
        // If there is no free room in queue, then replace
        // first message in queue by new SMsg

        if (q.cSize == q.cFilled)
        {
            pSQueue->pSMsg[q.cBegin] = SMsg;
            q.cBegin++;
            if (q.cBegin == q.cSize) q.cBegin = 0;

            OS_Flags.bEventError = 1;
            goto EXIT;
        }

        //------------------------------------------------------
        // There is a free room in queue.
        // Add new message at end of queue.

        temp = q.cBegin + q.cFilled;
        if (temp >= q.cSize) temp -= q.cSize;
        pSQueue->pSMsg[temp] = SMsg;
        q.cFilled++;

    EXIT:
        pSQueue->Q = q;
    }

//------------------------------------------------------------------------------
#endif  // (OS_ENABLE_INT_QUEUE) && !defined(_OS_Squeue_Send_I_DEFINED)
//------------------------------------------------------------------------------









/*
 ********************************************************************************
 *                                                                              *
 *  OST_SMSG _OS_Squeue_Get (OST_SQUEUE *pSQueue)                               *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  service OS_AcceptSQueue)                                    *
 *                                                                              *
 *                  Get first simple message from queue. Before calling this    *
 *                  function be sure that queue is not empty (OS_AcceptSQueue   *
 *                  does it automatically). After execution this function first *
 *                  message will be deleted from queue.                         *
 *                                                                              *
 *  parameters:     pSQueue      - pointer to queue descriptor                  *
 *                                                                              *
 *  on return:      first message from queue                                    *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                                                                              *
 ********************************************************************************
 */


//------------------------------------------------------------------------------
#if !defined(_OS_Squeue_Get_DEFINED)
//------------------------------------------------------------------------------

    OST_SMSG _OS_Squeue_Get (OST_SQUEUE *pSQueue)
    {
        OST_QUEUE_CONTROL   q;
        __osa_word       temp;
        OST_SMSG            smsg_temp;

        q = pSQueue->Q;
        temp = q.cBegin;
        q.cBegin++;

        if (q.cBegin >= q.cSize)    q.cBegin = 0;

        q.cFilled--;
        pSQueue->Q = q;

        smsg_temp = pSQueue->pSMsg[temp];

        return  smsg_temp;
    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_Queue_Get_DEFINED)
//------------------------------------------------------------------------------




/*
 ********************************************************************************
 *                                                                              *
 *  OST_SMSG _OS_Squeue_Get_I (OST_SQUEUE *pSQueue)                             *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  service OS_Squeue_Accept_I)                                 *
 *                                                                              *
 *                  Get first simple message from queue. Before calling this    *
 *                  function be sure that queue is not empty (OS_AcceptSQueue   *
 *                  does it automatically). After execution this function first *
 *                  message will be deleted from queue.                         *
 *                                                                              *
 *  parameters:     pSQueue      - pointer to queue descriptor                  *
 *                                                                              *
 *  on return:      first message from queue                                    *
 *                                                                              *
 *  Overloaded in:  -                                                           *
 *                                                                              *
 ********************************************************************************
 */


//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT_QUEUE) && !defined(_OS_Squeue_Get_I_DEFINED)
//------------------------------------------------------------------------------

    OST_SMSG _OS_Squeue_Get_I (OST_SQUEUE *pSQueue)
    {
        OST_QUEUE_CONTROL   q;
        __osa_word temp;

        q = pSQueue->Q;
        temp = q.cBegin;
        q.cBegin++;

        if (q.cBegin >= q.cSize)    q.cBegin = 0;

        q.cFilled--;
        pSQueue->Q = q;

        return  pSQueue->pSMsg[temp];
    }

//------------------------------------------------------------------------------
#endif  // defined(OS_ENABLE_INT_QUEUE) && !defined(_OS_Queue_Get_DEFINED)
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#endif  // defined(OS_ENABLE_SQUEUE)
//------------------------------------------------------------------------------





















//------------------------------------------------------------------------------
#if defined(OS_ENABLE_QUEUE)
//------------------------------------------------------------------------------

/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_Queue_Send (OST_QUEUE *pQueue, OST_MSG Msg)                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  service _OS_Queue_Send)                                     *
 *                                                                              *
 *                  Adds message into queue of pointers to messages. Deletes    *
 *                  first message if there is no free room to add new message.  *
 *                  Service OS_Queue_Send before adding new message checks for  *
 *                  free room. Thus messages will not deleted accidentally.     *
 *                                                                              *
 *                                                                              *
 *  parameters:     pQueue      - pointer to queue descriptor                   *
 *                  Msg        - pointer to  message to be added                *
 *                                                                              *
 *  on return:      OS_IsEventError() return 1, if first message was pushed out *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                                                                              *
 ********************************************************************************
 */


//------------------------------------------------------------------------------
#if !defined(_OS_Queue_Send_DEFINED)
//------------------------------------------------------------------------------

    void _OS_Queue_Send (OST_QUEUE *pQueue, OST_MSG Msg)
    {

        OST_QUEUE_CONTROL   q;
        __osa_word temp;

        q = pQueue->Q;
        OS_Flags.bEventError = 0;

        //------------------------------------------------------
        // If there is no free room in queue, then replace
        // first message in queue by new SMsg

        if (q.cSize == q.cFilled)
        {
            pQueue->pMsg[q.cBegin] = Msg;
            q.cBegin++;
            if (q.cBegin == q.cSize) q.cBegin = 0;

            OS_Flags.bEventError = 1;
            goto EXIT;
        }


        //------------------------------------------------------
        // There is a free room in queue.
        // Add new message at end of queue.

        temp = q.cBegin + q.cFilled;
        if (temp >= q.cSize) temp -= q.cSize;
        pQueue->pMsg[temp] = Msg;
        q.cFilled++;

    EXIT:

        pQueue->Q = q;
    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_Queue_Send_DEFINED)
//------------------------------------------------------------------------------








/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_Queue_Send_I (OST_QUEUE *pQueue, OST_MSG Msg)                      *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Copy of _OS_Queue_Send to be called from interrupt          *
 *                                                                              *
 *  parameters:     pQueue      - pointer to queue descriptor                   *
 *                  Msg        - pointer to message to be added                 *
 *                                                                              *
 *  on return:      OS_IsEventError() return 1, if first message was pushed out *
 *                                                                              *
 *  Overloaded in:  -                                                           *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT_QUEUE) && !defined(_OS_Queue_Send_I_DEFINED)
//------------------------------------------------------------------------------

    void _OS_Queue_Send_I (OST_QUEUE *pQueue, OST_MSG Msg)
    {

        OST_QUEUE_CONTROL   q;
        __osa_word temp;

        q = pQueue->Q;
        OS_Flags.bEventError = 0;

        //------------------------------------------------------
        // If there is no free room in queue, then replace
        // first message in queue by new SMsg

        if (q.cSize == q.cFilled)
        {
            pQueue->pMsg[q.cBegin] = Msg;
            q.cBegin++;
            if (q.cBegin == q.cSize) q.cBegin = 0;

            OS_Flags.bEventError = 1;
            goto EXIT;
        }

        //------------------------------------------------------
        // There is a free room in queue.
        // Add new message at end of queue.

        temp = q.cBegin + q.cFilled;
        if (temp >= q.cSize) temp -= q.cSize;
        pQueue->pMsg[temp] = Msg;
        q.cFilled++;

    EXIT:

        pQueue->Q = q;
    }

//------------------------------------------------------------------------------
#endif  // (OS_ENABLE_INT_QUEUE) && !defined(_OS_Queue_Send_I_DEFINED)
//------------------------------------------------------------------------------









/*
 ********************************************************************************
 *                                                                              *
 *  OST_MSG _OS_Queue_Get (OST_QUEUE *pQueue)                                    *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  service OS_AcceptQueue)                                     *
 *                                                                              *
 *                  Get first pointer to message from queue. Before calling this*
 *                  function be sure that queue is not empty (OS_AcceptQueue does*
 *                  it automatically). After execution this function first      *
 *                  message will be deleted from queue.                         *
 *                                                                              *
 *  parameters:     pQueue      - pointer to queue descriptor                   *
 *                                                                              *
 *  on return:      first message from queue                                    *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if !defined(_OS_Queue_Get_DEFINED)
//------------------------------------------------------------------------------

    OST_MSG _OS_Queue_Get (OST_QUEUE *pQueue)
    {
        OST_QUEUE_CONTROL   q;
        __osa_word       temp;
        OST_MSG             msg_temp;


        q = pQueue->Q;
        temp = q.cBegin;
        q.cBegin++;

        if (q.cBegin >= q.cSize)    q.cBegin = 0;

        q.cFilled--;
        pQueue->Q = q;

        msg_temp = pQueue->pMsg[temp];

        return  msg_temp;
    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_Queue_Get_DEFINED)
//------------------------------------------------------------------------------


/*
 ********************************************************************************
 *                                                                              *
 *  OST_MSG _OS_Queue_Get (OST_QUEUE *pQueue)                                    *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  service OS_Queue_Accept_I)                                   *
 *                                                                              *
 *                  Get first pointer to message from queue. Before calling this*
 *                  function be sure that queue is not empty (OS_AcceptQueue does*
 *                  it automatically). After execution this function first      *
 *                  message will be deleted from queue.                         *
 *                                                                              *
 *  parameters:     pQueue      - pointer to queue descriptor                   *
 *                                                                              *
 *  on return:      first message from queue                                    *
 *                                                                              *
 *  Overloaded in:  -                                                            *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT_QUEUE) && !defined(_OS_Queue_Get_I_DEFINED)
//------------------------------------------------------------------------------

    OST_MSG _OS_Queue_Get_I (OST_QUEUE *pQueue)
    {
        OST_QUEUE_CONTROL   q;
        __osa_word temp;

        q = pQueue->Q;
        temp = q.cBegin;
        q.cBegin++;

        if (q.cBegin >= q.cSize)    q.cBegin = 0;

        q.cFilled--;
        pQueue->Q = q;

        return  pQueue->pMsg[temp];
    }

//------------------------------------------------------------------------------
#endif  // defined(OS_ENABLE_INT_QUEUE) && !defined(_OS_Queue_Get_DEFINED)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif  // OS_ENABLE_QUEUE
//------------------------------------------------------------------------------

