/*
 ***********************************************************************************************
 *                                                                                              
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 
 *                                                                                              
 *  OSA is distributed under BSD license (see license.txt)                                      
 *                                                                                              
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              
 *                  http://picosa.narod.ru                                                      
 *                                                                                              
 *----------------------------------------------------------------------------------------------
 *                                                                                              
 *  File:           osa_picc18.c                                                                
 *                                                                                              
 *  Compilers:      HT-PICC18 STD                                                               
 *                                                                                              
 *  Programmer:     Timofeev Victor                                                             
 *                  osa@pic24.ru, testerplus@mail.ru                                            
 *                                                                                              
 *  Description:    PICC18 specific functions overloads                                         
 *                                                                                              
 *  History:        21.01.2009                                                                  
 *                  21.03.2009 -    OS_DI/OS_RI calls deleted from _OS_Csem_Signal              
 *                                  Bug fixed: OS_RI added into _OS_Dtimer_Delete               
 *                  01.04.2009 -    All processor specific definitions moved from osa.c into    
 *                                  this file                                                   
 *                     07.2009 -    1. _OS_Task_Create removed since it was identical to osa.c  
 *                                  2. _OS_Dtimer_xxx moved into osa.c                          
 *                  03.11.2009 -    3. Bug fixed: Added bReady=1 in OS_Check_Event when bTimeout
 *                                  4. Increased speed of OS_CheckEvent for non-priority mode
 *                  10.12.2009 -    Added services OS_DI_INT and OS_RI_INT
 *                                                                                              
 ***********************************************************************************************
 */



#define OS_Init_ProcSpec()      // Empty macro for PICC18








/************************************************************************************************
 *                                                                                              *
 *                           K E R N E L   F U N C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************/


void _OS_ClrReadySetClrCanContinue (void);  /* see description below    */



/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPoint (void)                                                *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to sheduler with saving current task return point    *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPoint (void)
{
    _OS_SET_FSR_CUR_TASK();
    _OS_SavePC();
    _OS_bTaskReady = 1;
    asm("    global _OS_SchedRetPointNoSave   ");
    asm("_OS_SchedRetPointNoSave:             ");
    _PIC18_POP();
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReady (void)                                                  *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag and return to the sheduler with seaving   *
 *                  task return point                                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

asm("   global  _OS_ClearReady");
asm("_OS_ClearReady:     ");
asm("   clrf    __OS_temp, c    ");
asm("   global  __OS_ClrReadySetClrCanContinue  ");
asm("   goto    __OS_ClrReadySetClrCanContinue  ");


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReadySetCanContinue (void)                                    *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag, set bCanContinue flag and return to the  *
 *                  sheduler with seaving task return point                     *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

asm("   global  _OS_ClearReadySetCanContinue");
asm("_OS_ClearReadySetCanContinue:     ");
asm("   setf    __OS_temp, c    ");
asm("   global  __OS_ClrReadySetClrCanContinue  ");
asm("   goto    __OS_ClrReadySetClrCanContinue  ");



/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClrReadySetClrCanContinue (void)                                   *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Addition function. Called only by GOTO from:                *
 *                      OS_ClearReady()                                         *
 *                      OS_ClearReadySetCanContinue()                           *
 *                  Stack has pointer of task who called theese services.       *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void _OS_ClrReadySetClrCanContinue (void)
{


    OS_Flags.bEventError = 0;

    _OS_SET_FSR_CUR_TASK();
    _OS_SavePC();

    _OS_bTaskReady = 0;

    #if defined(OS_ENABLE_TTIMERS)

        if (!_OS_bTaskDelay) _OS_bTaskCanContinue = 0;  /*100210.3*/
        //*100210.3*_OS_bTaskCanContinue = 0;

        if (_OS_temp & 0x01) _OS_bTaskCanContinue = 1;

    #endif

}
















/************************************************************************************************
 *                                                                                              *
 *  MAKE FSR POINTER TO OS_CUR_TASK                                                             *
 *                                                                                              *
 *  (for code reducing)                                                                         *
 *                                                                                              *
 ************************************************************************************************/

    asm("   global __OS_SET_FSR_CUR_TASK        ");
    asm("__OS_SET_FSR_CUR_TASK:                 ");
    _PIC18_ERRATA_NOP();
    asm("       movf        __OS_CurTask+0, w, c");
    asm("       movwf       __fsr + 0, c        ");
#if OS_BANK_TASKS == 0
    asm("       clrf        __fsr + 1, c        ");
#else
    asm("       movf        __OS_CurTask+1, w, c");
    asm("       movwf       __fsr + 1, c        ");
#endif
    asm("       return                          ");



















/*
 ************************************************************************************************
 *                                                                                              *
 *                            C O U N T I N G    S E M A P H O R E S                            *
 *                                                                                              *
 ************************************************************************************************
 */


//------------------------------------------------------------------------------
#ifdef OS_ENABLE_CSEM
//------------------------------------------------------------------------------


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_Csem_Signal (OST_CSEM *pCSem)                                      *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  Increase counting semaphore. Set EventError if csem is FF...                *
 *                                                                              *
 *                                                                              *
 *  parameters:     pCSem - pointer to counting semaphore                       *
 *                                                                              *
 *  on return:      OS_IsEventError()                                           *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_Csem_Signal_DEFINED
//------------------------------------------------------------------------------

void _OS_Csem_Signal (OST_CSEM *pCSem)
{
    OS_Flags.bEventError = 0;               // Counting semaphore overflow bit

    //------------------------------------------------------------------------------
    //
    #if OS_CSEM_SIZE == 1

        _fsr = (_FSR_TYPE)pCSem;
        _indf++;
        if (!_indf)
        {
            _indf--;
            OS_Flags.bEventError = 1;   // Set overflow bit
        }
    //------------------------------------------------------------------------------
    // For two- and four-bytes semaphores
    //
    #else
        (*pCSem)++;
        if (!*pCSem) {
            (*pCSem) = -1;
            OS_Flags.bEventError = 1;   // Set overflow bit
        }
    #endif
    //------------------------------------------------------------------------------

}


//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT_CSEM)
//------------------------------------------------------------------------------

    #define _OS_Csem_Signal_I_DEFINED

    void _OS_Csem_Signal_I (OST_CSEM *pCSem)
    {
        OS_Flags.bEventError = 0;               // Counting semaphore overflow bit
    
        //------------------------------------------------------------------------------
        //
        #if OS_CSEM_SIZE == 1
    
            _fsr = (_FSR_TYPE)pCSem;
            _indf++;
            if (!_indf)
            {
                _indf--;
                OS_Flags.bEventError = 1;   // Set overflow bit
            }
        //------------------------------------------------------------------------------
        // For two- and four-bytes semaphores
        //
        #else
            (*pCSem)++;
            if (!*pCSem) {
                (*pCSem) = -1;
                OS_Flags.bEventError = 1;   // Set overflow bit
            }
        #endif
        //------------------------------------------------------------------------------
    
    }

//------------------------------------------------------------------------------
#endif      // OS_ENABLE_INT_CSEM
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
#endif      // OS_INABLE_CSEM
//------------------------------------------------------------------------------



























/*
 ************************************************************************************************
 *                                                                                              *
 *                                   D E L A Y S   I N   T A S K                                *
 *                                                                                              *
 ************************************************************************************************
 */

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_TTIMERS
//------------------------------------------------------------------------------

/*
 ********************************************************************************
 *                                                                              *
 *   void _OS_InitDelay (_OST_TTIMER Delay)                                     *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel)                 *
 *                                                                              *
 *                  Init task timer delay.Set timer variable in task descriptor,*
 *                  sets bit bDelay and clears bit bCanContinue.                *
 *                                                                              *
 *                  _OS_CurTask must point to descriptor.                       *
 *                                                                              *
 *                                                                              *
 *  parameters:     Delay   - time of delay in system ticks                     *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_InitDelay_DEFINED
//------------------------------------------------------------------------------

void _OS_InitDelay (_OST_TTIMER Delay)
{

    _OS_bTaskCanContinue = 0;
    _OS_bTaskDelay = 0;         // Clear bDelay to prevent timer changing in interrupt
    _OS_SET_FSR_CUR_TASK();
    _indf = *((near char*)&OS_state);

    if (Delay)
    {

        Delay ^= -1;
        Delay ++;

        _OS_CurTask->Timer = Delay;

        _OS_bTaskDelay = 1;
        _OS_bTaskReady = 1;
        _OS_SET_FSR_CUR_TASK();
        _indf = *((near char*)&OS_state);

    }
}

//------------------------------------------------------------------------------
#endif  // #ifdef OS_ENABLE_TTIMERS
//------------------------------------------------------------------------------













































/************************************************************************************************
 *                                                                                              *
 *                            S Y S T E M   F U N C T I O N S                                   *
 *                                                                                              *
 ************************************************************************************************/











/*
 ********************************************************************************
 *                                                                              *
 *  char OS_DI (void)                                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Disable interrupt with GIE saving                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      char - bit 7 = previous GIEH value,                         *
 *                         bit 6 = previous GIEL value                          *
 *                         bits 0..5 = 0                                        *
 *                                                                              *
 ********************************************************************************
 */

//-------------------------------------------------------------------------------
#define OS_DI_DEFINED
//-------------------------------------------------------------------------------

    asm("   global _OS_DI           ");
    asm("_OS_DI:                    ");
    asm("   dw      0xFFFF          ");
    asm("   movf    __intcon, w, c  ");
    asm("   andlw   0xC0            ");
    asm("   bcf     __intcon, 7, c  ");
    asm("   btfsc   __rcon, 7, c    ");
    asm("   bcf     __intcon, 6, c  ");
    asm("   return                  ");
    asm("   dw      0xFFFF          ");

//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT)
//------------------------------------------------------------------------------

    asm("   global _OS_DI_INT       ");
    asm("_OS_DI_INT:                ");
    asm("   dw      0xFFFF          ");
    asm("   movf    __intcon, w, c  ");
    asm("   andlw   0xC0            ");
    asm("   bcf     __intcon, 7, c  ");
    asm("   btfsc   __rcon, 7, c    ");
    asm("   bcf     __intcon, 6, c   ");
    asm("   movwf   __OS_temp_I, c  ");
    asm("   return                  ");
    asm("   dw      0xFFFF          ");

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


/*
 ********************************************************************************
 *                                                                              *
 *  char OS_RI (void)                                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Restore saved by OS_DI() GIEx states                        *
 *                                                                              *
 *  parameters:     char - bit 7 = previos GIE value, bits 0..6 = 0             *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

//-------------------------------------------------------------------------------
#define OS_RI_DEFINED
//-------------------------------------------------------------------------------

    asm("   global  _OS_RI          ");
    asm("_OS_RI:                    ");
    asm("   dw      0xFFFF          ");
    asm("   btfsc   __wreg, 7, c     ");
    asm("   bsf     __intcon, 7, c   ");
    asm("   btfsc   __rcon, 7, c     ");
    asm("   btfss   __wreg, 6, c     ");
    asm("   bra     _RI_EXIT        ");
    asm("   bsf     __intcon, 6, c   ");
    asm("_RI_EXIT:                  ");
    asm("   return                  ");
    asm("   dw      0xFFFF          ");


//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT)
//------------------------------------------------------------------------------

    asm("   global  _OS_RI_INT          ");
    asm("_OS_RI_INT:                    ");
    asm("   dw      0xFFFF              ");
    asm("   btfsc   __OS_temp_I, 7, c   ");
    asm("   bsf     __intcon, 7, c      ");
    asm("   btfsc   __rcon, 7, c        ");
    asm("   btfss   __OS_temp_I, 6, c   ");
    asm("   bra     _RI_INT_EXIT        ");
    asm("   bsf     __intcon, 6, c      ");
    asm("_RI_INT_EXIT:                  ");
    asm("   return                      ");
    asm("   dw      0xFFFF              ");

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------





/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_CheckEvent (char bEvent)                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  waiting services)                                           *
 *                                                                              *
 *                  Check condition of bEvent. Accordint to bEvent value and    *
 *                  current state of bReady, bDelay and bCanContinue flags,     *
 *                  task becames ready to execute or still remains in waiting   *
 *                  mode.                                                       *
 *                                                                              *
 *  parameters:     bEvent - zero, or non-zero condition                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_CheckEvent_DEFINED
//------------------------------------------------------------------------------

void _OS_CheckEvent (__osa_word bEvent)
{
    //------------------------------------------------------------------------------
    #ifdef OS_DISABLE_PRIORITY                  // *91103.4*
    //------------------------------------------------------------------------------
    
        OS_Flags.bTimeout = 0;
        _OS_SET_FSR_CUR_TASK();
    
        if (bEvent)
        {
            #ifdef OS_ENABLE_TTIMERS

                _OS_bTaskDelay = 0; // Clear bDelay in temporary variable OS_state
                _indf &= ~0x10;     // Clear bDelay in task descriptor

            #endif
            return;
        }
    
        #ifdef OS_ENABLE_TTIMERS
    
            if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
            {
                OS_Flags.bTimeout = 1;
                return;
            }
    
        #endif
    
        _PIC18_POP();
        
    //------------------------------------------------------------------------------
    #else
    //------------------------------------------------------------------------------
    
        OS_Flags.bTimeout = 0;
        _OS_SET_FSR_CUR_TASK();
    
        if (bEvent)
        {
            if (_OS_bTaskReady) 
            {
                #ifdef OS_ENABLE_TTIMERS
    
                    _OS_bTaskDelay = 0; // Clear bDelay in temporary variable OS_state
                    _indf &= ~0x10;     // Clear bDelay in task descriptor
    
                #endif
    
                return;
            }
    
            _OS_bTaskReady = 1;
    
        } else {
            _OS_bTaskReady = 0;
        }
    
        #ifdef OS_ENABLE_TTIMERS
    
            if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
            {
                _OS_bTaskReady = 1;                         // *91103.3*
                OS_Flags.bTimeout = 1;
                return;
            }
    
        #endif
    
        _PIC18_POP();

    //------------------------------------------------------------------------------
    #endif
    //------------------------------------------------------------------------------
}
















