/*
 ***********************************************************************************************
 *                                                                                              
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 
 *                                                                                              
 *  OSA is distributed under BSD license (see license.txt)                                      
 *                                                                                              
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              
 *                  http://picosa.narod.ru                                                      
 *                                                                                              
 *----------------------------------------------------------------------------------------------
 *                                                                                              
 *  File:           osa_picc18.h                                                                
 *                                                                                              
 *  Compilers:      HT-PICC18 STD                                                               
 *                                                                                              
 *  Programmer:     Timofeev Victor                                                             
 *                  osa@pic24.ru, testerplus@mail.ru                                            
 *                                                                                              
 *  Description:    PICC18 specific definitions                                                 
 *                                                                                              
 *  History:        21.01.2009                                                                  
 *                  01.04.2009 -    All processor specific definitions moved from osa.h into    
 *                                  this file                                                   
 *                     07.2009 -    1. tempFSR1 deleted from OST_TCB                            
 *                                  2. OST_TCB_SIZE in assembler defines (see below) decresed   
 *                                     by 1 becuase of note 1 (tempFSR1)                        
 *                                                                                              
 ***********************************************************************************************
 */


#ifndef __OSAPICC18_H__
#define __OSAPICC18_H__


#if _HTC_EDITION_ == __PRO__
//#define __OSAPICCPRO__
//#warning "PRO!!!!"
#endif



/************************************************************************************************
 *                                                                                              *
 *     Integer types definitions                                                                * 
 *                                                                                              *
 ************************************************************************************************/

typedef unsigned char   __osa_uint8;
typedef unsigned int    __osa_uint16;
typedef unsigned long   __osa_uint32;
typedef bit             __osa_bool;

typedef __osa_uint8     __osa_word;
#define _OST_INT_MASK    7
#define _OST_INT_SHIFT   3
#define _OST_INT_SIZE    8



/************************************************************************************************
 *                                                                                              * 
 *     Location keywords                                                                        * 
 *                                                                                              * 
 ************************************************************************************************/

#define NEAR            near
#define bank0           near
#define bank1           bdata
#define bank2           bdata
#define bank3           bdata
#define __osa30_near
#define _OS_CUR_FLAGS_IN_OS_state   // Work with tasks state througth OS_state variable




/************************************************************************************************
 *                                                                                              * 
 *     Processor specified definitions for system flags                                         * 
 *     (see OST_SYSTEM_FLAGS and OST_TCB definition in file "OSA.h")                            * 
 *                                                                                              * 
 ************************************************************************************************/

#define _OST_SYSTEM_FLAGS_PROC_SPEC                                                             \
                                                                                                \
        /*5*/   __osa_word   bTimeout    : 1; /* Event waiting timeout                */     \
        /*6*/   __osa_word   bGIE_CTemp  : 1; /* Saved GIEH while in critical section */     \
        /*7*/   __osa_word   bGIEL_CTemp : 1; /* Saved GIEL while in critical section */     \

#define _OST_TCB_PROC_SPEC                                                                      \


/************************************************************************************************
 *                                                                                              *
 *     Registers definitions                                                                    *
 *                                                                                              *
 ************************************************************************************************/

static volatile near unsigned int _fsr      @ 0xFE9;
static volatile near char         _indf     @ 0xFEF;
static volatile near char         _postinc  @ 0xFEE;
static volatile near char         _postdec  @ 0xFED;
static volatile near char         _preinc   @ 0xFEC;
static volatile near char         _plusw    @ 0xFEB;

static volatile near unsigned char _fsr1l   @ 0xFE1;

static volatile near char         _pcl      @ 0xFF9;
static volatile near char         _pclath   @ 0xFFA;
static volatile near char         _pclatu   @ 0xFFB;
static volatile near char         _status   @ 0xFD8;
static volatile near char         _tosl     @ 0xFFD;
static volatile near char         _tosh     @ 0xFFE;
static volatile near char         _tosu     @ 0xFFF;
static volatile near char         _bsr      @ 0xFE0;
static volatile near char         _wreg     @ 0xFE8;
static volatile near char         _intcon   @ 0xFF2;
static volatile near char         _rcon     @ 0xFD0;


static          near bit    _giel       @ ((unsigned)&_intcon*8)+6;
static          near bit    _gie        @ ((unsigned)&_intcon*8)+7;
static          near bit    _ipen       @ ((unsigned)&_rcon*8)+7;
static volatile near bit    _carry      @ ((unsigned)&_status*8)+0;
static volatile near bit    _zero       @ ((unsigned)&_status*8)+2;


/************************************************************************************************
 *                                                                                              *
 *     Constants and types                                                                      *
 *                                                                                              *
 ************************************************************************************************/

#undef OS_SMSG_SIZE
#define OS_SMSG_SIZE    sizeof(OS_SMSG_TYPE)


//------------------------------------------------------------------------------

typedef unsigned int                _FSR_TYPE;

//------------------------------------------------------------------------------

#if defined(_ROMSIZE) && (_ROMSIZE <= 0x10000)

    typedef unsigned int                OST_CODE_POINTER;
    #define OS_CODE_POINTER_SIZE        2

#else

    typedef unsigned long               OST_CODE_POINTER;
    #define OS_CODE_POINTER_SIZE        4

#endif



//------------------------------------------------------------------------------

/************************************************************************************************
 *                                                                                              *
 *     Platform macros                                                                          *
 *                                                                                              *
 ************************************************************************************************/

#define _PIC18_POP()            asm("      pop     ");
#define _OS_POP18()             _PIC18_POP()

#if defined(_ROMSIZE) && (_ROMSIZE <= 0x4000)
#define _PIC18_ERRATA_NOP()
#else
#define _PIC18_ERRATA_NOP()     asm("\t    dw 0xFFFF     ")
#endif


#define OS_CLRWDT()             asm(" clrwdt ");
#define OS_ClrWdt()             asm(" clrwdt ");
#define OS_Clrwdt()             asm(" clrwdt ");
#define OS_SLEEP()              asm(" sleep  ");
#define OS_Sleep()              asm(" sleep  ");










/************************************************************************************************
 *                                                                                              *
 *     Context switching macros                                                                 *
 *                                                                                              *
 ************************************************************************************************/


//------------------------------------------------------------------------------
// Save/restore PC macros
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#if OS_CODE_POINTER_SIZE == 2
//------------------------------------------------------------------------------

    #define _OS_SetPC()     asm("   movf    __postinc, w, c     ");         \
                            asm("   movff   __postinc, __pclath ");         \
                            asm("   movwf   __pcl, c            ");


    #define _OS_SavePC()    asm("   movff   __tosl, __preinc    ");         \
                            asm("   movff   __tosh, __preinc    ");

//------------------------------------------------------------------------------
#else
//------------------------------------------------------------------------------

    #define _OS_SetPC()     asm("   movf    __postinc, w, c     ");         \
                            asm("   movff   __postinc, __pclath ");         \
                            asm("   movff   __postinc, __pclatu ");         \
                            asm("   movwf   __pcl, c            ");


    #define _OS_SavePC()    asm("   movff   __tosl, __preinc    ");         \
                            asm("   movff   __tosh, __preinc    ");         \
                            asm("   movff   __tosu, __preinc    ");

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------



/*
 ********************************************************************************
 *                                                                              *
 *  _OS_JumpToTask()                                                            *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:        jump indirectly from kernel (OS_Sched) into task        *
 *                                                                              *
 ********************************************************************************
 */

#define _OS_JumpToTask()                                                                        \
    {                                                                                           \
        asm("    bra     _OS_call_indirrect         ");                                         \
                                                                                                \
        asm("_OS_jump_indirrect:                    ");                                         \
                                                                                                \
        _OS_SET_FSR_CUR_TASK();                                                                 \
        *((near char*)&OS_state) = _postinc;                                                    \
        _OS_SetPC();                                                                            \
                                                                                                \
        asm("_OS_call_indirrect:                    ");                                         \
                                                                                                \
        asm("   rcall   _OS_jump_indirrect          ");                                         \
        _OS_SET_FSR_CUR_TASK();                                                                 \
        asm("   movlw   0xD7                        "); /*  except bReady and bCanContinue*/    \
        asm("   andwf   __indf, f, c                ");                                         \
        asm("   movlw   0x28                        "); /*  bReady and bCanContinue*/           \
        asm("   andwf   _OS_state, w, c             ");                                         \
        asm("   iorwf   __indf, f, c                ");                                         \
    }


#define _OS_SaveRetAddr()





/************************************************************************************************
 *                                                                                              *
 *     Return from task to OS_Sched macros                                                      *
 *                                                                                              *
 ************************************************************************************************/


/*
 ********************************************************************************
 *                                                                              *
 *   Switch control routines:                                                   *
 *                                                                              *
 *    _OS_RETURN_SAVE()                   - exit from task and save context     *
 *                                                                              *
 *    _OS_RETURN_NO_SAVE()                - exit withowt saving return address  *
 *                                                                              *
 *    _OS_CLEAR_READY()                   - exit with saving address and        *
 *                                          clearing bReady                     *
 *                                                                              *
 *    _OS_CLEAR_READY_SET_CANCONTINUE()   - exit without saving address and     *
 *                                          clearing bReady and bCanContinue    *
 *                                                                              *
 ********************************************************************************
 */


extern void OS_SchedRetPoint (void);
extern void OS_SchedRetPointNoSave (void);
extern void OS_ClearReady (void);
extern void OS_ClearReadySetCanContinue (void);

extern void  _OS_SET_FSR_CUR_TASK (void);


//------------------------------------------------------------------------------
#define _OS_RETURN_SAVE()           {                                           \
                                        OS_SchedRetPoint();                     \
                                        _PIC18_ERRATA_NOP();                    \
                                    }                                           \

//------------------------------------------------------------------------------
#define _OS_CLEAR_READY()           {                                           \
                                        OS_ClearReady();                        \
                                        _PIC18_ERRATA_NOP();                    \
                                    }                                           \

//------------------------------------------------------------------------------
#define _OS_CLEAR_READY_SET_CANCONTINUE()                                       \
                                    {                                           \
                                        OS_ClearReadySetCanContinue();          \
                                        _PIC18_ERRATA_NOP();                    \
                                    }                                           \

//------------------------------------------------------------------------------
#define _OS_RETURN_NO_SAVE()        {                                           \
                                        asm(" global _OS_SchedRetPointNoSave"); \
                                        OS_SchedRetPointNoSave();               \
                                        _PIC18_ERRATA_NOP();                    \
                                    }                                           \

//------------------------------------------------------------------------------








//______________________________________________________________________________
/******************************************************************************************
 *
 * MACROS FOR OLD STYLE STATIC TIMERS WORK
 *
 ******************************************************************************************/

#define __OS_Timer8Work(TIMER_ID)                                                               \
    {                                                                                           \
        if ((OS_Timeouts[(TIMER_ID)>>3]&(1<<((TIMER_ID)&7))))                                   \
        {                                                                                       \
            asm("\t    global   _OS_Timers8   ");                                               \
            asm("\t    movlb    (_OS_Timers8 + (" #TIMER_ID")) >> 8 ");                         \
            asm("\t    incfsz   (_OS_Timers8 + (" #TIMER_ID")) & 0xFF, f    ");                 \
            asm("\t    bra      _TIMER_NEXT_8_"#TIMER_ID"_     ");                              \
             OS_Timeouts[(TIMER_ID)>>3]&=~(1<<((TIMER_ID)&7));                                  \
            asm("_TIMER_NEXT_8_"#TIMER_ID"_: ");                                                \
        }                                                                                       \
    }


#define __OS_Timer16Work(TIMER_ID)                                                              \
    {                                                                                           \
        if ((OS_Timeouts[(TIMER_ID+_OS_TIMER16_POS)>>3]&(1<<((TIMER_ID+_OS_TIMER16_POS)&7))))   \
        {                                                                                       \
            asm("\t    global   _OS_Timers16  ");                                               \
            asm("\t    movlb    (_OS_Timers16 + (" #TIMER_ID"*2)) >> 8 ");                      \
            asm("\t    infsnz   (_OS_Timers16 + (" #TIMER_ID"*2)+0) & 0xFF, f    ");            \
            asm("\t    incfsz   (_OS_Timers16 + (" #TIMER_ID"*2)+1) & 0xFF, f    ");            \
            asm("\t    bra      _TIMER_NEXT_16_"#TIMER_ID"_     ");                             \
             OS_Timeouts[(TIMER_ID+_OS_TIMER16_POS)>>3]&=~(1<<((TIMER_ID+_OS_TIMER16_POS)&7));  \
            asm("_TIMER_NEXT_16_"#TIMER_ID"_: ");                                               \
        }                                                                                       \
    }


#define __OS_Timer24Work(TIMER_ID)                                                              \
    {                                                                                           \
        if ((OS_Timeouts[(TIMER_ID+_OS_TIMER24_POS)>>3]&(1<<((TIMER_ID+_OS_TIMER24_POS)&7))))   \
        {                                                                                       \
            asm("\t    global   _OS_Timers24  ");                                               \
            asm("\t    movlb    (_OS_Timers24 + (" #TIMER_ID"*2)) >> 8 ");                      \
            asm("\t    infsnz   (_OS_Timers24 + (" #TIMER_ID"*2)+0) & 0xFF, f    ");            \
            asm("\t    incfsz   (_OS_Timers24 + (" #TIMER_ID"*2)+1) & 0xFF, f    ");            \
            asm("\t    bra      _TIMER_NEXT_24_"#TIMER_ID"_     ");                             \
             OS_Timeouts[(TIMER_ID+_OS_TIMER24_POS)>>3]&=~(1<<((TIMER_ID+_OS_TIMER24_POS)&7));  \
            asm("_TIMER_NEXT_24_"#TIMER_ID"_: ");                                               \
        }                                                                                       \
    }


#define __OS_Timer32Work(TIMER_ID)                                                              \
    {                                                                                           \
        if ((OS_Timeouts[(TIMER_ID+_OS_TIMER32_POS)>>3]&(1<<((TIMER_ID+_OS_TIMER32_POS)&7))))   \
        {                                                                                       \
            asm("\t    global   _OS_Timers32  ");                                               \
            asm("\t    movlb    (_OS_Timers32 + (" #TIMER_ID"*4)) >> 8 ");                      \
            asm("\t    incf     (_OS_Timers32 + (" #TIMER_ID"*4)+0) & 0xFF, f    ");            \
            asm("\t    bnc      _TIMER_NEXT_32_"#TIMER_ID"_     ");                             \
            asm("\t    movlw    0     ");                                                       \
            asm("\t    addwfc   (_OS_Timers32 + (" #TIMER_ID"*4)+1) & 0xFF, f    ");            \
            asm("\t    addwfc   (_OS_Timers32 + (" #TIMER_ID"*4)+2) & 0xFF, f    ");            \
            asm("\t    addwfc   (_OS_Timers32 + (" #TIMER_ID"*4)+3) & 0xFF, f    ");            \
            asm("\t    bnc      _TIMER_NEXT_32_"#TIMER_ID"_     ");                             \
             OS_Timeouts[(TIMER_ID+_OS_TIMER32_POS)>>3]&=~(1<<((TIMER_ID+_OS_TIMER32_POS)&7));  \
            asm("_TIMER_NEXT_32_"#TIMER_ID"_: ");                                               \
        }                                                                                       \
    }


//______________________________________________________________________________
/******************************************************************************************
 *
 * MACROS FOR WORK WITH STATIC TIMERS
 *
 ******************************************************************************************/


/**/
//------------------------------------------------------------------------------
#ifdef OS_ENABLE_STIMERS
//------------------------------------------------------------------------------

#if OS_STIMER_SIZE == 4
    #define __OS_StimerBank()                               \
            asm("global __OS_Stimers");                       \
            asm("movlb __OS_Stimers>>8");                     \
            asm("OS_S0 set low __OS_Stimers");                \
            asm("OS_S1 set low __OS_Stimers+1");              \
            asm("OS_S2 set low __OS_Stimers+2");              \
            asm("OS_S3 set low __OS_Stimers+3");              \
            asm("movlw 0");
#else
    #define __OS_StimerBank()                               \
            asm("global __OS_Stimers");                       \
            asm("movlb __OS_Stimers>>8");                     \
            asm("OS_S0 set low __OS_Stimers");                \
            asm("OS_S1 set low __OS_Stimers+1");
#endif


//------------------------------------------------------------------------------
#if     OS_STIMER_SIZE == 1
//------------------------------------------------------------------------------

    #define __OS_StimerWork(STIMER_ID)                  \
        asm("   btfsc OS_S0+"#STIMER_ID",7");           \
        asm("   incf OS_S0+"#STIMER_ID"");              \


    #define __OS_StimersWorkSize()                      \
        _fsr = (_FSR_TYPE)_OS_Stimers - 1;              \
        _wreg = OS_STIMERS;                             \
        do {                                            \
            if (_preinc & 0x80) _indf++;                \
        } while (--_wreg);                              \

//------------------------------------------------------------------------------
#elif   OS_STIMER_SIZE == 2
//------------------------------------------------------------------------------

    #define __OS_StimerWork(STIMER_ID)                  \
        asm("btfss OS_S1+("#STIMER_ID"*2),7");          \
        asm("bra $+6");                                 \
        asm("infsnz OS_S0+("#STIMER_ID"*2)");           \
        asm("incf OS_S1+("#STIMER_ID"*2)");             \

    #define __OS_StimersWorkSize()                      \
        _fsr = (_FSR_TYPE)_OS_Stimers + OS_STIMERS*2;     \
        do {                                            \
            _postdec;                                   \
            if (_postdec & 0x80)                        \
            {                                           \
                if (!++_postinc) _indf++;               \
                _postdec;                               \
            }                                           \
            asm("   movlw   __OS_Stimers  ");             \
            asm("   xorwf   __fsr, w, c ");             \
        } while (!ZERO);                                \

//------------------------------------------------------------------------------
#elif   OS_STIMER_SIZE == 4
//------------------------------------------------------------------------------

    #define __OS_StimerWork(STIMER_ID)                  \
        asm("btfss OS_S3+("#STIMER_ID"*4),7");          \
        asm("bra $+10");                                \
        asm("incf OS_S0+("#STIMER_ID"*4)");             \
        asm("addwfc OS_S1+("#STIMER_ID"*4)");           \
        asm("addwfc OS_S2+("#STIMER_ID"*4)");           \
        asm("addwfc OS_S3+("#STIMER_ID"*4)");           \


    #define __OS_StimersWorkSize()                          \
        _fsr = (_FSR_TYPE)_OS_Stimers;                        \
        do {                                                \
            asm("   movlw   3           ");                 \
            asm("   addwf   __fsr, f, c ");                 \
            if (_indf & 0x80)                               \
            {                                               \
                asm("   subwf   __fsr, f, c     ");         \
                asm("   incf    __postinc, f, c");          \
                asm("   movlw   0             ");           \
                asm("   addwfc  __postinc, f, c");          \
                asm("   addwfc  __postinc, f, c");          \
                asm("   addwfc  __indf, f, c");             \
            }                                               \
            _postinc;                                       \
            _wreg = OS_STIMERS * 4 + (char)&_OS_Stimers;      \
            asm("   xorwf   __fsr, w, c     ");             \
        } while (!ZERO);


//------------------------------------------------------------------------------
#endif  // OS_STIMER_SIZE
//------------------------------------------------------------------------------
/**/ 

//------------------------------------------------------------------------------
#endif  // OS_ENABLE_STIMERS
//------------------------------------------------------------------------------
 
 
 
 
 
 
 
 
 
 
 
//______________________________________________________________________________
/******************************************************************************************
 *
 * MACROS FOR WORK WITH TASK TIMERS
 *
 ******************************************************************************************/


//------------------------------------------------------------------------------
#if   OS_CODE_POINTER_SIZE == 2
//------------------------------------------------------------------------------

    #if     OS_TTIMER_SIZE == 1
        #define ASM_OST_TCB_SIZE_CONST      asm("TCB_SIZE set 4");
    #elif   OS_TTIMER_SIZE == 2
        #define ASM_OST_TCB_SIZE_CONST      asm("TCB_SIZE set 5");
    #elif   OS_TTIMER_SIZE == 4
        #define ASM_OST_TCB_SIZE_CONST      asm("TCB_SIZE set 7");
    #endif

//------------------------------------------------------------------------------
#elif OS_CODE_POINTER_SIZE == 4
//------------------------------------------------------------------------------

    #if     OS_TTIMER_SIZE == 1
        #define ASM_OST_TCB_SIZE_CONST      asm("TCB_SIZE set 6");
    #elif   OS_TTIMER_SIZE == 2
        #define ASM_OST_TCB_SIZE_CONST      asm("TCB_SIZE set 7");
    #elif   OS_TTIMER_SIZE == 4
        #define ASM_OST_TCB_SIZE_CONST      asm("TCB_SIZE set 9");
    #endif


//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#ifdef OS_ENABLE_TTIMERS
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Set BSR for work with task timers
#define     ASM_SET_BANK    asm(" movlb high(_OS_TaskVars)");       \
                            asm("DELAY_BIT set 4");                 \
                            asm("OS_TSKVAR set low _OS_TaskVars");  \


//------------------------------------------------------------------------------
// Expression compiles correctly in this case only:
//        asm("   movwf   ("#TASK_ID" +1 )* TCB_SIZE + low(_OS_TaskVars) - 3 ");
//------------------------------------------------------------------------------
#if   OS_TTIMER_SIZE == 1
//------------------------------------------------------------------------------

    #define __OS_TtimerWork(TASK_ID)                                \
        asm("btfss "#TASK_ID"*TCB_SIZE+OS_TSKVAR,DELAY_BIT");       \
        asm("bra $+6");                                             \
        asm("infsnz ("#TASK_ID"+1)*TCB_SIZE+OS_TSKVAR-1");          \
        asm("bcf "#TASK_ID"*TCB_SIZE+OS_TSKVAR,DELAY_BIT");         \

    #define __OS_TtimersWorkSize()                                  \
        ASM_OST_TCB_SIZE_CONST;                                     \
        ASM_SET_BANK;                                               \
        _fsr = (_FSR_TYPE)OS_TaskVars;                              \
        do {                                                        \
            asm("   movlw   TCB_SIZE");                             \
            if (((near OST_TASK_STATE*)(&_indf))->bDelay)           \
            {                                                       \
                asm("   movlw   TCB_SIZE - 1");                     \
                asm("   addwf   __fsr, f, c     ");                 \
                asm("   incf    __postinc, f, c ");                 \
                asm("   movlw   0               ");                 \
                if (CARRY)                                          \
                {                                                   \
                    asm("   movlw   TCB_SIZE    ");                 \
                    asm("   subwf   __fsr, f, c ");                 \
                    ((near OST_TASK_STATE*)(&_indf))->bDelay = 0;   \
                }                                                   \
            }                                                       \
            asm("   addwf   __fsr, f, c ");                         \
            _wreg = OS_TASKS*(OS_TTIMER_SIZE + OS_CODE_POINTER_SIZE + 1) + (char)&OS_TaskVars;  \
            asm("   xorwf   __fsr, w, c         ");                 \
        } while (!ZERO);                                            \

//------------------------------------------------------------------------------
#elif OS_TTIMER_SIZE == 2
//------------------------------------------------------------------------------

    #define __OS_TtimerWork(TASK_ID)                                \
        asm("btfsc "#TASK_ID"*TCB_SIZE+OS_TSKVAR,DELAY_BIT");       \
        asm("incfsz ("#TASK_ID"+1)*TCB_SIZE+OS_TSKVAR-2");          \
        asm("bra $+6");                                             \
        asm("infsnz ("#TASK_ID"+1)*TCB_SIZE+OS_TSKVAR-1");          \
        asm("bcf "#TASK_ID"*TCB_SIZE+OS_TSKVAR,DELAY_BIT");         \

    #define __OS_TtimersWorkSize()                                  \
        ASM_OST_TCB_SIZE_CONST;                                     \
        ASM_SET_BANK;                                               \
        _fsr = (_FSR_TYPE)OS_TaskVars;                              \
        do {                                                        \
            asm("   movlw   TCB_SIZE");                             \
            if (((near OST_TASK_STATE*)(&_indf))->bDelay)           \
            {                                                       \
                asm("   movlw   TCB_SIZE - 2");                     \
                asm("   addwf   __fsr, f, c     ");                 \
                asm("   incf    __postinc, f, c ");                 \
                asm("   movlw   0               ");                 \
                asm("   addwfc  __postinc, f, c ");                 \
                if (CARRY)                                          \
                {                                                   \
                    asm("   movlw   TCB_SIZE    ");                 \
                    asm("   subwf   __fsr, f, c ");                 \
                    ((near OST_TASK_STATE*)(&_indf))->bDelay = 0;   \
                }                                                   \
            }                                                       \
            asm("   addwf   __fsr, f, c ");                         \
            _wreg = OS_TASKS*(OS_TTIMER_SIZE + OS_CODE_POINTER_SIZE + 1) + (char)&OS_TaskVars;  \
            asm("   xorwf   __fsr, w, c         ");                 \
        } while (!ZERO);                                            \


//------------------------------------------------------------------------------
#elif OS_TTIMER_SIZE == 4
//------------------------------------------------------------------------------

    #define __OS_TtimerWork(TASK_ID)                                \
        asm("btfsc "#TASK_ID"*TCB_SIZE+OS_TSKVAR,DELAY_BIT");       \
        asm("incfsz ("#TASK_ID"+1)*TCB_SIZE+OS_TSKVAR-4");          \
        asm("bra $+12");                                            \
        asm("infsnz ("#TASK_ID"+1)*TCB_SIZE+OS_TSKVAR-3");          \
        asm("incfsz ("#TASK_ID"+1)*TCB_SIZE+OS_TSKVAR-2");          \
        asm("bra $+6");                                             \
        asm("infsnz ("#TASK_ID" + 1)*TCB_SIZE+OS_TSKVAR-1");        \
        asm("bcf "#TASK_ID"*TCB_SIZE+OS_TSKVAR,DELAY_BIT");


    #define __OS_TtimersWorkSize()                                  \
        ASM_OST_TCB_SIZE_CONST;                                     \
        ASM_SET_BANK;                                               \
        _fsr = (_FSR_TYPE)OS_TaskVars;                              \
        do {                                                        \
            asm("   movlw   TCB_SIZE");                             \
            if (((near OST_TASK_STATE*)(&_indf))->bDelay)           \
            {                                                       \
                asm("   movlw   TCB_SIZE - 4");                     \
                asm("   addwf   __fsr, f, c     ");                 \
                asm("   incf    __postinc, f, c ");                 \
                asm("   movlw   0               ");                 \
                asm("   addwfc  __postinc, f, c ");                 \
                asm("   addwfc  __postinc, f, c ");                 \
                asm("   addwfc  __postinc, f, c ");                 \
                if (CARRY)                                          \
                {                                                   \
                    asm("   movlw   TCB_SIZE    ");                 \
                    asm("   subwf   __fsr, f, c ");                 \
                    ((near OST_TASK_STATE*)(&_indf))->bDelay = 0;   \
                }                                                   \
            }                                                       \
            asm("   addwf   __fsr, f, c ");                         \
            _wreg = OS_TASKS*(OS_TTIMER_SIZE + OS_CODE_POINTER_SIZE + 1) + (char)&OS_TaskVars;\
            asm("   xorwf   __fsr, w, c         ");                 \
        } while (!ZERO);                                            \


//------------------------------------------------------------------------------
#endif      // if sizeof(OST_TIMER)
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#endif      // OS_ENABLE_TTIMERS
//------------------------------------------------------------------------------

/******************************************************************************************/









/******************************************************************************************
 *
 *  WORK WITH DYNAMIC TIMERS
 *
 ******************************************************************************************/

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------

#define _OS_DtimersWork_DEFINED

#define _OS_IndfTimerTimeout       ((NEAR OST_DTIMER_FLAGS*)(&_indf))->bTimeout
#define _OS_IndfTimerActive        ((NEAR OST_DTIMER_FLAGS*)(&_indf))->bActive
#define _OS_IndfTimerRun           ((NEAR OST_DTIMER_FLAGS*)(&_indf))->bRun
#define _OS_IndfTimerNextEnable    ((NEAR OST_DTIMER_FLAGS*)(&_indf))->bNextEnable
#define _OS_PostincTimerNextEnable ((NEAR OST_DTIMER_FLAGS*)(&_postinc))->bNextEnable



//------------------------------------------------------------------------------
#if   OS_DTIMER_SIZE == 1
//------------------------------------------------------------------------------

    #define OS_INC_DTIMER()                                                             \
        {                                                                               \
            asm("   movlw   3                   ");                                     \
            asm("   addwf   __fsr, f, c         ");  /* FSR0L can't overflow here */    \
            asm("   incf    __postdec, f, c     ");                                     \
            asm("   movf    __postdec, w, c     ");                                     \
            asm("   movf    __postdec, w, c     ");                                     \
            if (_carry) _OS_IndfTimerTimeout = 1;                                       \
        }                                                                               \


//------------------------------------------------------------------------------
#elif OS_DTIMER_SIZE == 2
//------------------------------------------------------------------------------

    #define OS_INC_DTIMER()                                                             \
        {                                                                               \
            _OS_DTimers.Flags.bTimeout = 0;                                             \
            asm("   movlw   3                   ");                                     \
            asm("   addwf   __fsr, f, c         ");  /* FSR0L can't overflow here */    \
            asm("   movlw   0 ");                                                       \
            asm("   incf    __postinc, f, c     ");                                     \
            asm("   addwfc  __postdec, f, c     ");                                     \
            if (_carry) _OS_DTimers.Flags.bTimeout = 1;                                 \
            asm("   movlw   3                   ");                                     \
            asm("   subwf   __fsr, f, c         "); /* FSR0L can't overflow here */     \
            if (_OS_DTimers.Flags.bTimeout)                                             \
                _OS_IndfTimerTimeout = 1;                                               \
        }                                                                               \

//------------------------------------------------------------------------------
#elif OS_DTIMER_SIZE == 4
//------------------------------------------------------------------------------

    #define OS_INC_DTIMER()                                                             \
        {                                                                               \
            _OS_DTimers.Flags.bTimeout = 0;                                             \
            asm("   movlw   3                   ");                                     \
            asm("   addwf   __fsr, f, c         ");  /* FSR0L can't overflow here */    \
            asm("   movlw   0                   ");                                     \
            asm("   incf    __postinc, f, c     ");                                     \
            asm("   addwfc  __postinc, f, c     ");                                     \
            asm("   addwfc  __postinc, f, c     ");                                     \
            asm("   addwfc  __postdec, f, c     ");                                     \
            if (_carry) _OS_DTimers.Flags.bTimeout = 1;                                 \
            asm("   movlw   5                   ");                                     \
            asm("   subwf   __fsr, f, c         "); /* FSR0L can't overflow here */     \
            if (_OS_DTimers.Flags.bTimeout)                                             \
                _OS_IndfTimerTimeout = 1;                                               \
        }                                                                               \

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


#define __OS_DtimersWork()                              \
    {                                                   \
        _fsr = (_FSR_TYPE) &_OS_DTimers.Flags;          \
                                                        \
    REPEAT:                                             \
                                                        \
        if (_OS_PostincTimerNextEnable)                 \
        {                                               \
                                                        \
            /* Point FSR to next timer in list   */     \
                                                        \
            asm("   movf    __postinc, w, c        ");  \
            asm("   movff   __postdec, __fsr + 1   ");  \
            asm("   movwf   __fsr + 0, c            "); \
            if (!_OS_IndfTimerRun) goto REPEAT;         \
            OS_INC_DTIMER();                            \
                                                        \
            goto REPEAT;                                \
        }                                               \
    }



//------------------------------------------------------------------------------
#endif  // OS_ENABLE_DTIMER
//------------------------------------------------------------------------------









//------------------------------------------------------------------------------
#if defined(OS_ENABLE_QTIMERS)  /*100210.1*/
//------------------------------------------------------------------------------

#define _OS_IndfFTimerTimeout       ((NEAR OST_QTIMER_FLAGS*)(&_indf))->bTimeout
#define _OS_IndfFTimerActive        ((NEAR OST_QTIMER_FLAGS*)(&_indf))->bActive
#define _OS_IndfFTimerNextEnable    ((NEAR OST_QTIMER_FLAGS*)(&_indf))->bNextEnable

#define _OS_QTimersWork_DEFINED

//------------------------------------------------------------------------------
#if   OS_QTIMER_SIZE == 1
//------------------------------------------------------------------------------

    #define OS_IF_INC_QTIMER()                              \
        if (!++_indf)

    #define OS_CHECK_QTIMER()                               \
        asm("   movf    __indf, f, c    ");

//------------------------------------------------------------------------------
#elif OS_QTIMER_SIZE == 2
//------------------------------------------------------------------------------

    #define OS_IF_INC_QTIMER()                              \
        if (!++_postinc)                                    \
        if (!++_indf   )                                    \

    #define OS_CHECK_QTIMER()                               \
        asm("   movf    __postinc, w, c   ");               \
        asm("   iorwf   __indf, w, c   ");

//------------------------------------------------------------------------------
#elif OS_QTIMER_SIZE == 4
//------------------------------------------------------------------------------

    #define OS_IF_INC_QTIMER()                              \
        if (!++_postinc)                                    \
        if (!++_postinc)                                    \
        if (!++_postinc)                                    \
        if (!++_indf   )                                    \

    #define OS_CHECK_QTIMER()                               \
        asm("   movf    __postinc, w, c   ");               \
        asm("   iorwf   __postinc, w, c   ");               \
        asm("   iorwf   __postinc, w, c   ");               \
        asm("   iorwf   __indf   , w, c   ");               \



//------------------------------------------------------------------------------
#endif  // OS_QTIMER_SIZE
//------------------------------------------------------------------------------

#define _OS_QtimersWork_DEFINED


#define __OS_QtimersWork()                                      \
    if (_OS_QTimers.Flags.bNextEnable)                          \
    {                                                           \
        _fsr = ((_FSR_TYPE)&_OS_QTimers) + 1;                   \
        asm("   movf    __postinc, w, c        ");              \
        asm("   addlw   3                      ");              \
        asm("   movff   __indf, __fsr + 1      ");              \
        asm("   movwf   __fsr + 0, c           ");              \
        OS_IF_INC_QTIMER()                                      \
        do  {                                                   \
            _fsr -= (OS_QTIMER_SIZE + 2);                       \
            _OS_IndfFTimerTimeout = 1;                          \
            _OS_IndfFTimerActive  = 0;                          \
            if (!_OS_IndfFTimerNextEnable)                      \
            {                                                   \
                _OS_QTimers.Flags.bNextEnable = 0;              \
                break;                                          \
            }                                                   \
            asm("   movf    __preinc, w, c        ");           \
            asm("   movff   __preinc, __fsr + 1      ");        \
            asm("   movwf   __fsr + 0, c           ");          \
            _OS_QTimers.Next = (void*)_fsr;                     \
            _fsr += 3;                                          \
            OS_CHECK_QTIMER();                                  \
        } while (_zero);                                        \
    }

/**/
//------------------------------------------------------------------------------
#endif  // OS_ENABLE_QTIMERS
//------------------------------------------------------------------------------













/************************************************************************************************
 *                                                                                              * 
 *     Create, replace, reserve task specific services                                          * 
 *                                                                                              * 
 ************************************************************************************************/

/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

#define OS_Task_Replace(priority, TaskName)                                     \
{                                                                               \
    OS_Task_Reserve(TaskName);                                                  \
    *((NEAR __osa_word*)&OS_state) = priority | 0x48; /* bEnable  bReady */ \
    _OS_SET_FSR_CUR_TASK();                                                     \
    _postinc = priority | 0x48; /* bEnable  bReady */                          \
    _OS_SetIndfWord(TaskName);                                                  \
    _OS_Return();                                                               \
}


#define OS_Task_Replace_P(priority, TaskAddr)                                   \
{                                                                               \
    *((NEAR __osa_word*)&OS_state) = priority | 0x48; /* bEnable  bReady */ \
    _OS_SET_FSR_CUR_TASK();                                                     \
    _postinc = priority | 0x48; /* bEnable  bReady */                          \
    _OS_SetIndfWord((__osa_uint16)(TaskAddr));                                  \
    _OS_Return();                                                               \
}



/************************************************************************/
/*                                                                      */
/* Create task                                                          */
/*                                                                      */
/************************************************************************/

//--------------------------------------------------------------------------
#if defined(__OSAPICCPRO__)
//--------------------------------------------------------------------------

    #define OS_Task_Create(priority, TaskName)                          \
        {                                                               \
            _OS_Task_Create(priority, &TaskName);                       \
            asm("   incf    __fsr, f                ");                 \
            asm("   movlw   low(fp__"#TaskName")    ");                 \
            asm("   movwf   __indf                  ");                 \
            asm("   movlw   high(fp__"#TaskName")   ");                 \
            asm("   incf    __fsr, f                ");                 \
            asm("   movwf   __indf                  ");                 \
            if (_indf & 0x80) TaskName();                               \
        }

//--------------------------------------------------------------------------
#else
//--------------------------------------------------------------------------
    #define OS_Task_Create(priority, TaskName)                          \
        {                                                               \
            OS_Task_Reserve(TaskName);                                  \
            _OS_Task_Create(priority, (OST_CODE_POINTER)TaskName);      \
        }
//--------------------------------------------------------------------------
#endif
//--------------------------------------------------------------------------


/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/


#define OS_Task_Reserve(TaskName)                                       \
    {                                                                   \
        asm("\t global _main       ");                                  \
        asm("\t global _" #TaskName);                                   \
        asm("\t fncall _main,_" #TaskName);                             \
    }










/************************************************************************************************
 *                                                                                              *
 *                                     I N T E R R U P T S                                      * 
 *                                                                                              *
 ************************************************************************************************/

extern char OS_DI (void);
extern void OS_RI (char);
#define OS_EI()         { _gie = 1; if (_ipen) _giel = 1;}
#define OS_EIH()        { _gie = 1; }
#define OS_EIL()        { _giel = 1; }

//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT)
//------------------------------------------------------------------------------
    extern void OS_DI_INT (void);
    extern void OS_RI_INT (void);
//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


/************************************************************************************************
 *                                                                                              *
 *                                     C O N D I T I O N S                                      *
 *                                                                                              *
 ************************************************************************************************/

extern void _OS_CheckEvent (__osa_word);

#define _OS_CHECK_EVENT(event) _OS_CheckEvent(event)






#endif















