/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_winavr.h                                                                *    
 *                                                                                              *
 *  Compilers:      WinAVR                                                                      *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    OSA specific definitions for WinAVR                                         *
 *                                                                                              *
 *  History:        07.03.2010 -    File created                                                *
 *                                                                                              *
 ************************************************************************************************
 */

#ifndef __OSAWINAVR_H__
#define __OSAWINAVR_H__



/************************************************************************************************
 *                                                                                              *
 *     Integer types definitions                                                                *
 *                                                                                              *
 ************************************************************************************************/

typedef unsigned char   __osa_uint8;
typedef unsigned int    __osa_uint16;
typedef unsigned long   __osa_uint32;
typedef unsigned char   __osa_bool;

typedef __osa_uint8     __osa_word;
#define _OST_INT_MASK    7
#define _OST_INT_SHIFT   3
#define _OST_INT_SIZE    8



/************************************************************************************************
 *                                                                                              *
 *     Location keywords                                                                        *
 *                                                                                              *
 ************************************************************************************************/

#define NEAR
#define bank0
#define bank1
#define bank2
#define bank3
#define __osa30_near
#define _OS_CUR_FLAGS_IN_OS_state        // Work with tasks state througth OS_state



/************************************************************************************************
 *                                                                                              *
 *     Processor specified definitions for system flags                                         *
 *     (see OST_SYSTEM_FLAGS and OST_TCB definition in file "OSA.h")                            *
 *                                                                                              *
 ************************************************************************************************/

#define _OST_SYSTEM_FLAGS_PROC_SPEC                                                             \
                                                                                                \
        /*5*/   __osa_word   bTimeout    : 1; /* Event waiting timeout                */     \
        /*6*/   __osa_word   bGIE_CTemp  : 1; /* Saved GIEH while in critical section */     \

#define _OST_TCB_PROC_SPEC                                                                      \
                __osa_uint16    nY_Temp;         /* Saved R28:R29 */


#define OST_TASK_POINTER   OS_TASKS_BANK OST_TCB *

/************************************************************************************************
 *                                                                                              *
 *     Registers definitions                                                                    *
 *                                                                                              *
 ************************************************************************************************/

extern __osa_uint16 _OS_SP_Temp;


/************************************************************************************************
 *                                                                                              *
 *     Constants and types                                                                      *
 *                                                                                              *
 ************************************************************************************************/

#ifndef OS_SMSG_SIZE
#define OS_SMSG_SIZE    0   // We can not use sizeof() because it does not work with #ifdef directive
#endif

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

#ifdef __AVR_2_BYTE_PC__

    #define OST_CODE_POINTER        __osa_uint16
    #define OS_CODE_POINTER_SIZE    2

#else

    #define OST_CODE_POINTER        __osa_uint32
    #define OS_CODE_POINTER_SIZE    4

#endif





/************************************************************************************************
 *                                                                                              *
 *     Platform macros                                                                          *
 *                                                                                              *
 ************************************************************************************************/

#define _OS_POP()             asm(" pop r22 "); asm(" pop r23 ");

//#define _PIC18_ERRATA_NOP()


#define OS_CLRWDT()             asm("  wdr   ")
#define OS_SLEEP()              asm("  sleep   ")








/************************************************************************************************
 *                                                                                              *
 *     Context switching macros                                                                 *
 *                                                                                              *
 ************************************************************************************************/


//------------------------------------------------------------------------------
// Save/restore PC macros
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#if OS_CODE_POINTER_SIZE == 2
//------------------------------------------------------------------------------

    #define _OS_SetPC()                         \
        asm("   ld      r22, x+         ");     \
        asm("   sts     OS_state, r22   ");     \
        /* push task entry point into stack */  \
        asm("   ld      r22, x+         ");     \
        asm("   push    r22             ");     \
        asm("   ld      r22, x+         ");     \
        asm("   push    r22             ");     \
        /* load saved Y value               */  \
        asm("   ld      r28, x+         ");     \
        asm("   ld      r29, x+         ");     \
        asm(" SKIP:                     ");     \
    
    
    #define _OS_SavePC()                        \
        asm("   ld      r22, x+     ");         \
        /* get return address               */  \
        asm("   pop     r22         ");         \
        asm("   pop     r23         ");         \
        /* and save it into TCB             */  \
        asm("   st      x+, r23     ");         \
        asm("   st      x+, r22     ");         \
        /* save Y                           */  \
        asm("   st      x+, r28     ");         \
        asm("   st      x+, r29     ");
    
    #define _OS_Push()                          \
        /* push return address back to stack */ \
        asm("   push    r23         ");         \
        asm("   push    r22         ");         \
    

//------------------------------------------------------------------------------
#else 
//------------------------------------------------------------------------------

    #define _OS_SetPC()                         \
        asm("   ld      r22, x+         ");     \
        asm("   sts     OS_state, r22   ");     \
        /* push task entry point into stack */  \
        asm("   ld      r22, x+         ");     \
        asm("   push    r22             ");     \
        asm("   ld      r22, x+         ");     \
        asm("   push    r22             ");     \
        asm("   ld      r22, x+         ");     \
        asm("   push    r22             ");     \
        /* skip highes byte of dwaddress    */  \
        asm("   ld      r22, x+         ");     \
        /* load saved Y value               */  \
        asm("   ld      r28, x+         ");     \
        asm("   ld      r29, x+         ");     \
        asm(" SKIP:                     ");     \


    #define _OS_SavePC()                        \
        asm("   ld      r22, x+     ");         \
        /* get return address               */  \
        asm("   pop     r22         ");         \
        asm("   pop     r23         ");         \
        asm("   pop     r21         ");         \
        /* and save it into TCB             */  \
        asm("   st      x+, r21     ");         \
        asm("   st      x+, r23     ");         \
        asm("   st      x+, r22     ");         \
        /* write 0 to highest byte of dw addr */\
        asm("   ldi     r22, 0      ");         \
        asm("   st      x+, r22     ");         \
        /* save Y                           */  \
        asm("   st      x+, r28     ");         \
        asm("   st      x+, r29     ");

    #define _OS_Push()                          \
        /* push return address back to stack */ \
        asm("   push    r21         ");         \
        asm("   push    r23         ");         \
        asm("   push    r22         ");         \


//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------

#define     _OS_Store_SP()      _OS_SP_Temp = SP        // Save SP

// Restore SP

#define     _OS_Restore_SP()                \
    {                                       \
        _OS_temp = SREG;                    \
        asm("   cli              ");        \
        SP = _OS_SP_Temp;                   \
        SREG = _OS_temp;                    \
    }



/*
 ********************************************************************************
 *                                                                              *
 *  _OS_JumpToTask()                                                            *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:        jump indirectly from kernel (OS_Sched) into task        *
 *                                                                              *
 ********************************************************************************
 */

void __OS_JumpToTask (void) __attribute__ ((noinline));


#define _OS_JumpToTask()                                \
        asm("   push  r28     ");                       \
        asm("   push  r29     ");                       \
        __OS_JumpToTask();                              \


#define _OS_SaveRetAddr()                               \
    {                                                   \
        _OS_SET_FSR_CUR_TASK();                         \
        asm("   in      r24, 0x3F       ");             \
        asm("   cli                     ");             \
        asm("   ld      r23, x          ");             \
        asm("   andi    r23, 0xD7       ");             \
        asm("   lds     r22, OS_state   ");             \
        asm("   andi    r22, 0x28       ");             \
        asm("   or      r23, r22        ");             \
        asm("   st      x, r23          ");             \
        asm("   out     0x3F, r24       ");             \
        asm("   pop     r29             ");             \
        asm("   pop     r28             ");             \
    }






/************************************************************************************************
 *                                                                                              *
 *     Return from task to OS_Sched macros                                                      *
 *                                                                                              *
 ************************************************************************************************/


/*
 ********************************************************************************
 *                                                                              *
 *   Switch control routines:                                                   *
 *                                                                              *
 *    _OS_RETURN_SAVE()                   - exit from task and save context     *
 *                                                                              *
 *    _OS_RETURN_NO_SAVE()                - exit withowt saving return address  *
 *                                                                              *
 *    _OS_CLEAR_READY()                   - exit with saving address and        *
 *                                          clearing bReady                     *
 *                                                                              *
 *    _OS_CLEAR_READY_SET_CANCONTINUE()   - exit without saving address and     *
 *                                          clearing bReady and bCanContinue    *
 *                                                                              *
 ********************************************************************************
 */


extern void OS_SchedRetPoint            (void) __attribute__ ((noinline, returns_twice));;
extern void OS_SchedRetPointNoSave      (void) __attribute__ ((noinline, returns_twice));;
extern void OS_ClearReady               (void) __attribute__ ((noinline));;
extern void OS_ClearReadySetCanContinue (void) __attribute__ ((noinline));;

#define  _OS_SET_FSR_CUR_TASK()     { asm(" lds r26, _OS_CurTask "); asm(" lds r27, _OS_CurTask+1 ");}


//------------------------------------------------------------------------------
#define _OS_RETURN_SAVE()                       \
        OS_SchedRetPoint();                     \

//------------------------------------------------------------------------------
#define _OS_CLEAR_READY()                       \
        OS_ClearReady();                        \

//------------------------------------------------------------------------------
#define _OS_CLEAR_READY_SET_CANCONTINUE()       \
        OS_ClearReadySetCanContinue();          \

//------------------------------------------------------------------------------
#define _OS_RETURN_NO_SAVE()                    \
        OS_SchedRetPointNoSave();               \
//------------------------------------------------------------------------------











/************************************************************************************************
 *                                                                                              *
 *     Create, replace, reserve task specific services                                          *
 *                                                                                              *
 ************************************************************************************************/


/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

#define OS_Task_Replace(priority, TaskName)                                         \
    {                                                                               \
        *((NEAR __osa_word*)&OS_state) = priority | 0x48; /* bEnable  bReady */ \
        _OS_CurTask->State = OS_state;                                              \
        _OS_CurTask->pTaskPointer = (OST_CODE_POINTER)(&TaskName);                  \
        _OS_CurTask->nY_Temp = 0x8000;                                              \
        _OS_Return();                                                               \
    }

#define OS_Task_Replace_P(priority, TaskAddr)      /*100300.3*/                     \
    {                                                                               \
        *((NEAR __osa_word*)&OS_state) = priority | 0x48; /* bEnable  bReady */ \
        _OS_CurTask->State = OS_state;                                              \
        _OS_CurTask->pTaskPointer = (OST_CODE_POINTER)(TaskAddr);                   \
        _OS_CurTask->nY_Temp = 0x8000;                                              \
        _OS_Return();                                                               \
    }


/************************************************************************/
/*                                                                      */
/* Create task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Create(priority, TaskName)                          \
    {                                                               \
        _OS_Task_Create(priority, (OST_CODE_POINTER)&(TaskName));   \
    }


/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

// If task created indirrectly then it is nessesary to insert in
// the main():
// #pragma funcall main TaskName

#define OS_Task_Reserve(TaskName)       // *91210* Empty macro for this compiler



/************************************************************************************************
 *                                                                                              *
 *                                     I N T E R R U P T S                                      *
 *                                                                                              *
 ************************************************************************************************/

extern char OS_DI (void);
extern void OS_RI (char);
#define OS_EI()             sei()

//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT)
//------------------------------------------------------------------------------
    extern void OS_DI_INT (void);
    extern void OS_RI_INT (void);
//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


/************************************************************************************************
 *                                                                                              *
 *                                     C O N D I T I O N S                                      *
 *                                                                                              *
 ************************************************************************************************/

extern void _OS_CheckEvent (__osa_word) __attribute__ ((noinline, returns_twice));

#define _OS_CHECK_EVENT(event) _OS_CheckEvent(event)





#endif





