/*
 ***********************************************************************************************
 *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC
 *
 *  OSA is distributed under BSD license (see license.txt)
 *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro
 *                  http://picosa.narod.ru
 *
 *----------------------------------------------------------------------------------------------
 *
 *  File:           osa_ftimer.h
 *
 *  Programmer:     Timofeev Victor
 *                  osa@pic24.ru, testerplus@mail.ru
 *
 *  Description:    Services for fast timers.
 *
 *  History:        18.07.2009 -    File created
 *
 ***********************************************************************************************
 */

#ifdef OS_ENABLE_QTIMERS


_OST_QTIMER _OS_Qtimer_List  (_OST_QTIMER time, OST_QTIMER *ftimer);
//void        _OS_Qtimer_Count (void)


#define __OS_Qtimer_Add(ftimer, time)                                   \
    OSM_BEGIN {                                                         \
        _OS_Qtimer_List(time, &(ftimer));                               \
    } OSM_END                                                           \


#define __OS_Qtimer_Delete(ftimer)                                      \
    OSM_BEGIN {                                                         \
        _OS_QTimers.Flags.bListFunction = OS_QTIMER_LIST_FUNCTION_DELETE;\
        _OS_Qtimer_List(0, &(ftimer));                                  \
    } OSM_END                                                           \


#define OS_Qtimer_Check(ftimer)      ((ftimer).Flags.bTimeout)       // overflowed
#define OS_Qtimer_IsRun(ftimer)      ((ftimer).Flags.bActive)        // created

#define OS_Qtimer_Run(ftimer, time)                                     \
    OSM_BEGIN {                                                         \
        __OS_Qtimer_Add(ftimer, time);                                  \
    } OSM_END                                                           \


#define OS_Qtimer_Delete(ftimer)                                        \
    OSM_BEGIN {                                                         \
        __OS_Qtimer_Delete(ftimer);                                     \
    } OSM_END


#define OS_Qtimer_Break(ftimer)                                         \
    OSM_BEGIN {                                                         \
        __OS_Qtimer_Delete(ftimer);                                     \
        (ftimer).Flags.bTimeout = 1;                                    \
    } OSM_END



#define OS_Qtimer_Create(ftimer)                                        \
    OSM_BEGIN {                                                         \
        (ftimer).Flags.bActive  = 0;                                    \
        (ftimer).Flags.bTimeout = 1;                                    \
    } OSM_END


#define OS_Qtimer_Get(ftimer)                                           \
    (_OS_QTimers.Flags.bListGetTime = 1, _OS_Qtimer_List(0, &(ftimer))) \



#define OS_Qtimer_Wait(ftimer)                  \
    OSM_BEGIN {                                 \
        _OS_WAIT_EVENT(OS_Qtimer_Check(ftimer));\
    } OSM_END


#define OS_Qtimer_Delay(ftimer, delay)          \
    {                                           \
        OS_Qtimer_Run(ftimer, delay);           \
        OS_Qtimer_Wait(ftimer);                 \
    }

#endif  // OS_ENABLE_QTIMERS
