/*
 ***********************************************************************************************
 *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC
 *
 *  OSA is distributed under BSD license (see license.txt)
 *
 *  URL:        http://wiki.pic24.ru/doku.php/en/osa/ref/intro
 *              http://picosa.narod.ru
 *
 *----------------------------------------------------------------------------------------------
 *
 *  File:       osa_stimer.c
 *
 *  Programmer: Timofeev Victor
 *              osa@pic24.ru, testerplus@mail.ru
 *
 *  Definition: Functions for work with static timers
 *
 *  History:    21.01.2010  -   File created
 *
 ***********************************************************************************************
 */


/************************************************************************************************
 *                                                                                              *
 *                     W O R K   W I T H   S T A T I C   T I M E R S                            *
 *                                                                                              *
 ************************************************************************************************/

//-----------------------------------------------------------------
#ifdef  OS_ENABLE_STIMERS
#ifdef OS_STIMERS_ENABLE_ALLOCATION
//-----------------------------------------------------------------


/*
 ********************************************************************************
 *
 *  _OS_Stimer_GetFree
 *
 *------------------------------------------------------------------------------
 *
 *  description:    (Internal function called by system kernel througth service OS
 *                  Searches free static timer and returns it's ID
 *
 *
 *  parameters:     __osa_word bCreate - =0 - do not created timer, just check for
 *                                               aviability
 *                                          =1 - create timer and mark it asm used
 *
 *  on return:      none
 *
 *  Overloaded in:
 *
 ********************************************************************************
 */
/**/



//==============================================================================
#ifdef __OSAPICC18__
//==============================================================================

    __osa_uint8   _OS_Stimer_GetFree (__osa_word bCreate)
    {
        OS_Flags.bError = 0;
        _fsr = (_FSR_TYPE)_OS_StimersFree;
        _OS_temp = (OS_STIMERS >> 3) + 1;
        do
        {
            if (_indf)
            {
                if (!(bCreate & 1)) asm("   return ");

                asm("   movlw   low(__OS_StimersFree) "); // w = (position in array) * 8
                asm("   subwf   __fsr, w, c         ");
                asm("   rlcf    __wreg, f, c        ");
                asm("   rlcf    __wreg, f, c        ");
                asm("   rlcf    __wreg, w, c        ");
                asm("   andlw   0xF8                ");

                asm("   movff   __indf, __OS_tempH  "); // Search for first "1" in _indf
                asm("   movwf   __OS_temp, c        ");
                asm("   movlw   1                   ");
                asm("OS_ST_GF_REPEAT:               ");
                asm("   rrcf    __OS_tempH, f, c    ");
                asm("   bc      OS_ST_GF_OK         ");
                asm("   rlcf    __wreg, f, c        ");
                asm("   incf    __OS_temp, f, c     ");
                asm("   bra     OS_ST_GF_REPEAT     ");
                asm("OS_ST_GF_OK:                   ");

                asm("   xorwf  __indf, f, c         "); // Clear bit of selected timer
                asm("   movf   __OS_temp, w, c      "); // Return bit position (timer's ID)
                asm("   return                      ");
            }
            _fsr++;
        } while (--_OS_temp);

        OS_Flags.bError = 1;
        return 0xFF;
    }


    //void    _OS_Stimer_Free (__osa_uint8 ID)
    asm("   global  __OS_Stimer_Free            ");
    asm("__OS_Stimer_Free:                      ");
    asm("   dw      0xFFFF                      ");
    asm("   lfsr    0, __OS_StimersFree         ");
    asm("   movwf   __OS_temp, c                ");
    asm("   rrcf    __wreg, w, c                ");     //  wreg = ID / 8
    asm("   rrcf    __wreg, w, c                ");
    asm("   rrcf    __wreg, w, c                ");
    asm("   andlw   0x1F                        ");
    asm("   addwf   __fsr, f, c                 ");

    asm("   movlw   0x01                        ");     // W = 1 << (ID & 7)
    asm("   btfsc   __OS_temp, 2, c             ");
    asm("   swapf   __wreg, w, c                ");
    asm("   btfsc   __OS_temp, 1, c             ");
    asm("   rlcf    __wreg, w, c                ");
    asm("   btfsc   __OS_temp, 1, c             ");
    asm("   rlcf    __wreg, w, c                ");
    asm("   btfsc   __OS_temp, 0, c             ");
    asm("   rlcf    __wreg, w, c                ");

    asm("   iorwf   __indf, f, c                ");     // Free timer
    asm("   return                              ");


//==============================================================================
#elif   defined(__OSAMCC18__)
//==============================================================================
    __osa_uint8   _OS_Stimer_GetFree (__osa_word bCreate)
    {
        OS_Flags.bError = 0;
        _fsr = (_FSR_TYPE)_OS_StimersFree;
        _OS_temp = (OS_STIMERS >> 3) + 1;
        do
        {
            if (_indf)
            {
                if (!(bCreate & 1)) return 0;

                _asm
                    movlw   _OS_StimersFree
                    subwf   _fsr, 0, 0
                    rlcf    _wreg, 1, 0
                    rlcf    _wreg, 1, 0
                    rlcf    _wreg, 0, 0
                    andlw   0xF8

                    movff   _indf, _OS_tempH  // Search for first "1" in _indf
                    movwf   _OS_temp, 0
                    movlw   1
                 OS_ST_GF_REPEAT:
                    rrcf    _OS_tempH, 1, 0
                    bc      OS_ST_GF_OK
                    rlcf    _wreg, 1, 0
                    incf    _OS_temp, 1, 0
                    bra     OS_ST_GF_REPEAT
                 OS_ST_GF_OK:

                    xorwf  _indf, 1, 0         // Clear bit of selected timer

                _endasm
                return _OS_temp;

            }
            _fsr++;
        } while (--_OS_temp);

        OS_Flags.bError = 1;
        return 0xFF;
    }


    void    _OS_Stimer_Free (__osa_uint8 ID)
    {
        _OS_temp = ID;
        _asm
        __OS_Stimer_Free:            
            lfsr    0, _OS_StimersFree
            movf    _OS_temp, 0, 0     
            rrcf    _wreg, 0, 0          //  wreg = ID / 8
            rrcf    _wreg, 0, 0     
            rrcf    _wreg, 0, 0     
            andlw   0x1F             
            addwf   _fsr, 1, 0      
        
            movlw   0x01                  // W = 1 << (ID & 7)
            btfsc   _OS_temp, 2, 0  
            swapf   _wreg, 0, 0     
            btfsc   _OS_temp, 1, 0  
            rlcf    _wreg, 0, 0     
            btfsc   _OS_temp, 1, 0  
            rlcf    _wreg, 0, 0     
            btfsc   _OS_temp, 0, 0  
            rlcf    _wreg, 0, 0     
            iorwf   _indf, 1, 0          // Free timer
        _endasm
        return   ;                
    }


//==============================================================================
#elif   defined(__OSAPICC16__)
//==============================================================================

    __osa_uint8   _OS_Stimer_GetFree (__osa_word bCreate)
    {
        char i;

        OS_Flags.bError = 0;
        _fsr = (_FSR_TYPE)_OS_StimersFree;
        _SET_STIMER_IRP();
        _OS_temp = (OS_STIMERS >> 3) + 1;
        i = 0;
        do
        {
            asm("   movf    __indf, w               ");
            asm("   btfsc   __status, 2             ");
            asm("   goto    OS_ST_GF_NEXT           ");

                if (!(bCreate & 1)) asm("   return ");

                asm("   movwf   __OS_tempH          ");
                asm("   movlw   __OS_StimersFree & 0xFF"); // w = (position in array) * 8
                asm("   subwf   __fsr, w            ");
                asm("   movwf   __OS_temp           ");
                asm("   rlf     __OS_temp, f        ");
                asm("   rlf     __OS_temp, f        ");
                asm("   rlf     __OS_temp, w        ");
                asm("   andlw   0xF8                ");

                asm("   clrf    __OS_Stimer_GetFree$i");  // Search for first "1" in _indf
                asm("   incf    __OS_Stimer_GetFree$i, f");
                asm("OS_ST_GF_REPEAT:               ");
                asm("   rrf     __OS_tempH, f       ");
                asm("   btfsc   __status, 0         ");
                asm("   goto    OS_ST_GF_OK         ");
                asm("   rlf     __OS_Stimer_GetFree$i, f");
                asm("   addlw   0x01                ");
                asm("   goto     OS_ST_GF_REPEAT    ");
                asm("OS_ST_GF_OK:                   ");

                asm("   movwf   __OS_temp            ");
                asm("   comf    __OS_Stimer_GetFree$i, w");
                asm("   andwf   __indf, f            "); // Clear bit of selected timer
                asm("   movf    __OS_temp, w         "); // Return bit position (timer's ID)
                asm("   return                      ");

            asm("OS_ST_GF_NEXT:                     ");
            _fsr++;
        } while (--_OS_temp);

        OS_Flags.bError = 1;
        return 0xFF;
    }


    //void    _OS_Stimer_Free (__osa_uint8 ID)
    asm("   global  __OS_Stimer_Free            ");
    asm("__OS_Stimer_Free:                      ");
    asm("   clrf    __status                    ");
    asm("   movwf   __OS_tempH                  ");

    asm("   movwf   __OS_temp                   ");     // fsr = &_OS_Stimer_Free[ID >> 3]
    asm("   rrf     __OS_temp, f                ");
    asm("   rrf     __OS_temp, f                ");
    asm("   rrf     __OS_temp, w                ");
    asm("   andlw   0x1F                        ");
    asm("   addlw   __OS_StimersFree & 0xFF       ");
    asm("   movwf   __fsr                       ");
    _SET_STIMER_IRP();

    asm("   clrf    __OS_temp                   ");     // W = 1 << (IS & 7)
    asm("   incf    __OS_temp, f                ");
    asm("   btfsc   __OS_tempH, 2               ");
    asm("   swapf   __OS_temp, f                ");
    asm("   btfsc   __OS_tempH, 0               ");     // CARRY = 0 now
    asm("   rlf     __OS_temp, f                ");     // CARRY = 0 now
    asm("   btfsc   __OS_tempH, 1               ");     // CARRY = 0 now
    asm("   rlf     __OS_temp, f                ");     // CARRY = 0 now
    asm("   btfsc   __OS_tempH, 1               ");     // CARRY = 0 now
    asm("   rlf     __OS_temp, f                ");     // CARRY = 0 now

    asm("   movf    __OS_temp, w                ");
    asm("   iorwf   __indf, f                   ");     // Free timer
    asm("   return                              ");


//==============================================================================
#elif   defined(__OSAPICC12__)
//==============================================================================


//------------------------------------------------------------------------------
// Macros for setting OS variables bank
//------------------------------------------------------------------------------

    //------------------------------------------------------------------------------
    #if     OS_BANK_STIMERS == 0
    //------------------------------------------------------------------------------

        #define _OS_SET_STIMER_IRP()
        #define _OS_SET_STIMER_IRP_0()

    //------------------------------------------------------------------------------
    #elif   OS_BANK_STIMERS == 1
    //------------------------------------------------------------------------------

        #define _OS_SET_STIMER_IRP()    asm("   bsf     __fsr, 5 ");
        #define _OS_SET_STIMER_IRP_0()  asm("   bcf     __fsr, 5 ");

    //------------------------------------------------------------------------------
    #elif   OS_BANK_STIMERS == 2
    //------------------------------------------------------------------------------

        #define _OS_SET_STIMER_IRP()    asm("   bsf     __fsr, 6 ");
        #define _OS_SET_STIMER_IRP_0()  asm("   bcf     __fsr, 6 ");

    //------------------------------------------------------------------------------
    #elif   OS_BANK_STIMERS == 3
    //------------------------------------------------------------------------------

        #define _OS_SET_STIMER_IRP()    asm("   bsf     __fsr, 5 ");    \
                                        asm("   bsf     __fsr, 6 ");

        #define _OS_SET_STIMER_IRP_0()  asm("   bcf     __fsr, 5 ");    \
                                        asm("   bcf     __fsr, 6 ");

    //------------------------------------------------------------------------------
    #endif
    //------------------------------------------------------------------------------




    __osa_uint8   _OS_Stimer_GetFree (__osa_word bCreate)
    {
        char i;

        OS_Flags.bError = 0;
        i = 0;
        _OS_temp = (OS_STIMERS >> 3) + 1;
        _fsr = (_FSR_TYPE)_OS_StimersFree;

        do
        {
            _OS_SET_STIMER_IRP();
            if (_indf)
            {
                asm("   movf    __indf, w                   ");
                _OS_SET_STIMER_IRP_0();
                asm("   movwf   __OS_temp                   ");

                asm("   btfss    ?a__OS_InitDelay, 0        ");
                asm("   goto    OS_ST_GF_RETURN             ");

                asm("   movf    __fsr, w                    ");
                asm("   clrf    __fsr                       "); // fsr used as a counter of bit position
                asm("   clrf    __OS_Stimer_GetFree$i       ");
                asm("   incf    __OS_Stimer_GetFree$i, f    ");
                asm("OS_ST_GF_REPEAT:                       ");
                asm("   rrf     __OS_temp, f                ");
                asm("   btfsc   __status, 0                 ");
                asm("   goto    OS_ST_GF_OK                 ");
                asm("   rlf     __OS_Stimer_GetFree$i, f    ");
                asm("   incf    __fsr, f                    ");
                asm("   goto     OS_ST_GF_REPEAT            ");
                asm("OS_ST_GF_OK:                           ");

                // Exchange WREG with FSR (saved fsr <-> bits counter)
                asm("   xorwf   __fsr, w                    ");
                asm("   xorwf   __fsr, f                    ");
                asm("   xorwf   __fsr, w                    ");
                _OS_PIC12_AND_7F();
                asm("   movwf   __OS_temp                   ");

                // Now i contains bit position, j contains bit mask
                asm("   movf    __OS_Stimer_GetFree$i, w    ");
                _OS_SET_STIMER_IRP();
                asm("   xorwf   __indf, f                   "); // Mark timer as "in use"

                asm("   movlw   __OS_StimersFree              ");
                asm("   subwf   __fsr, f                    ");
                asm("   rlf     __fsr, f                    ");
                asm("   rlf     __fsr, f                    ");
                asm("   rlf     __fsr, w                    ");
                asm("   andlw   0xF8                        ");

                asm("   clrf    __fsr                       ");

                asm("   addwf   __OS_temp, w                ");
                asm("OS_ST_GF_RETURN:                       ");
                return _OS_temp;
            }
            _fsr++;
            _OS_SET_STIMER_IRP_0();
        } while (--_OS_temp);

        OS_Flags.bError = 1;
        return 0xFF;
    }


    void    _OS_Stimer_Free (__osa_uint8 ID)
    {
        asm("   clrf    __OS_temp                   ");     // W = 1 << (IS & 7)
        asm("   incf    __OS_temp, f                ");
        asm("   btfsc   ?a__OS_Stimer_Free, 2       ");
        asm("   swapf   __OS_temp, f                ");

        if (ID & 1) _OS_temp <<= 1;
        asm("   btfsc   ?a__OS_Stimer_Free, 1       ");
        asm("   rlf     __OS_temp, f                ");     // CARRY = 0 now
        asm("   btfsc   ?a__OS_Stimer_Free, 1       ");
        asm("   rlf     __OS_temp, f                ");     // CARRY = 0 now

        _fsr = (_FSR_TYPE)_OS_StimersFree;
        _OS_SET_STIMER_IRP_0();

        asm("   rrf    ?a__OS_Stimer_Free, f        ");
        asm("   rrf    ?a__OS_Stimer_Free, f        ");
        asm("   rrf    ?a__OS_Stimer_Free, w        ");
        asm("   andlw   0x1F                        ");
        asm("   addwf   __fsr, f                    ");

        asm("   movf    __OS_temp, w                ");
        _OS_SET_STIMER_IRP();
        asm("   iorwf   __indf, f                   ");
        _fsr = 0;
    }


//==============================================================================
#else  // All other compilers
//==============================================================================

    __osa_uint8   _OS_Stimer_GetFree (__osa_word bCreate)
    {
        __osa_word n, i;
        __osa_word temp, mask;

        OS_Flags.bError = 0;

        for (n = 0; n <= OS_STIMERS >> 3; n++)
        {
            temp = _OS_StimersFree[n];
            if (temp)           // There is a free timer
            {
                if (!(bCreate & 1)) return 0;

                i = n << _OST_INT_SHIFT;
                mask = 1;
                while (!(temp & mask))
                {
                    mask <<= 1;
                    i++;
                }
                _OS_StimersFree[n] &= ~mask;
                return i;
            }
        }

        OS_Flags.bError = 1;
        return 0xFF;
    }




    void    _OS_Stimer_Free (__osa_uint8 ID)
    {
        _OS_StimersFree[ID >> _OST_INT_SHIFT] |= (1 << (ID & _OST_INT_MASK));
    }
    /**/

//==============================================================================
#endif  // Compiler type
//==============================================================================

//-----------------------------------------------------------------
#endif
#endif  //  OS_ENABLE_STIMERS
//-----------------------------------------------------------------



